<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output indent="yes" method="html"/>
<!--
<xsl:strip-space elements="*"/>
-->

<xsl:template match="translation_unit">
<HTML>
	<HEAD>
	   <TITLE><xsl:value-of select="@filename"/></TITLE>
	</HEAD>


	<BODY BGCOLOR="white" VLINK="blue"><OL START="1">
	<xsl:for-each select="function_definition | declaration">
	   
<!--
write the filename and put a line each files 
-->
       <xsl:variable name="current-file" select="@filename"/>
	   <xsl:variable name="pre-node" 
	            select="preceding-sibling::*[1]"/>
	   <xsl:if test="not($current-file=$pre-node/@filename)">
		  <BR/><BR/><BR/><BR/>
		  <A NAME="{$current-file}"></A>
	      <H2><xsl:value-of select="$current-file"/></H2>
		  <HR COLOR="gray"/>
	   </xsl:if>


       <xsl:if test="name()='function_definition'">
	      <BR/><BR/>
	   </xsl:if>
       <xsl:call-template name="LI_head"/>
	   <xsl:apply-templates select="."/>

	</xsl:for-each>   
    </OL></BODY>
</HTML>
</xsl:template>


<xsl:template name="LI_head">
   <xsl:param name="current_lineno">
      <xsl:call-template name="true_lineno"/>
   </xsl:param>
   <xsl:choose>
   <xsl:when test="$current_lineno=''">
      <xsl:text disable-output-escaping="yes"> &lt;LI> </xsl:text>
   </xsl:when>
   <xsl:otherwise>
      <xsl:text disable-output-escaping="yes"> &lt;LI VALUE=" </xsl:text>   
      <xsl:value-of select="$current_lineno"/>
      <xsl:text disable-output-escaping="yes"> "> </xsl:text>
   </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template name="LI_tail">
   <xsl:text disable-output-escaping="yes"> &lt;/LI> </xsl:text>
</xsl:template>

<xsl:template name="true_lineno">
  <xsl:for-each select="  descendant::identifier[1] 
                        | descendant::statement[1]">
     <xsl:if test="position()=1">
        <xsl:value-of select="@lineno"/>
     </xsl:if>
  </xsl:for-each>
</xsl:template>


<xsl:template name="indent">
   <xsl:param name="tab"> 1 </xsl:param>
   <xsl:if test="$tab = 1">
      <xsl:text> &#160;&#160;&#160;&#160; </xsl:text>
   </xsl:if>
   <xsl:if test="$tab > 1">
      <xsl:text> &#160;&#160;&#160;&#160; </xsl:text>
      <xsl:call-template name="indent">
	     <xsl:with-param name="tab">
		    <xsl:value-of select="$tab - 1"/>
		 </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
</xsl:template>

<xsl:template name="set-tab-">
   <xsl:call-template name="indent">
      <xsl:with-param name="tab">
         <xsl:value-of
		    select="count(ancestor::statement[@rhs='compound']) +
			        count(ancestor::statement[@rhs='if']) +
					count(ancestor::statement[@rhs='if-else'][1]) +
					count(ancestor::statement[@rhs='while']) +
					count(ancestor::statement[@rhs='for']) +
					count(ancestor::statement[@rhs='do-while']) +
					count(ancestor::statement[@rhs='switch']) +
					count(ancestor::statement[@rhs='case']) +
					count(ancestor::statement[@rhs='default'])-1"/>

      </xsl:with-param>
   </xsl:call-template>
</xsl:template>

<!--
			        count(ancestor::statement[@rhs='if']) +
					count(ancestor::statement[@rhs='if-else']) +
					count(ancestor::statement[@rhs='while']) +
					count(ancestor::statement[@rhs='for']) +
					count(ancestor::statement[@rhs='do-while']) +
					count(ancestor::statement[@rhs='switch']) +
					count(ancestor::statement[@rhs='case']) +
					count(ancestor::statement[@rhs='default'])-1"/>
-->


<xsl:template name="set-tab">
   <xsl:call-template name="set-tab-"/>
   <xsl:text> &#160;&#160;&#160;&#160; </xsl:text>
</xsl:template>

<xsl:template name="set-tab+">
   <xsl:call-template name="set-tab"/>
   <xsl:text> &#160;&#160;&#160;&#160; </xsl:text>
</xsl:template>


<xsl:template match="function_definition">
   <A NAME="{@id}"></A>
   <xsl:for-each select="*">
      <xsl:call-template name="declaration_specifiers"/>
	  <xsl:if test="name()='declarator'">
         <xsl:apply-templates select="."/>
		 <xsl:call-template name="LI_tail"/>
	  </xsl:if>
	  <xsl:if test="name()='declaration'">
	     <xsl:call-template name="LI_head"/>
         <xsl:text> &#160;&#160;&#160;&#160; </xsl:text>
		 <xsl:apply-templates select="."/>
	  </xsl:if>
	  <xsl:if test="name()='statement'">
	     <xsl:apply-templates select="."/>
	  </xsl:if>
   </xsl:for-each>
   <BR/><BR/>
</xsl:template>


<xsl:template match="declaration">
   <A NAME="{@id}"></A>
   <xsl:for-each select="*">
      <xsl:call-template name="declaration_specifiers"/>
   </xsl:for-each>
   <xsl:for-each select="declarator | initializer">
      <xsl:if test="name()='declarator'">
         <xsl:if test="position()=1">
	        <xsl:apply-templates select="."/>
	     </xsl:if>
         <xsl:if test="1 &lt; position()">
            <xsl:text>, </xsl:text>
	        <xsl:apply-templates select="."/>
         </xsl:if>
      </xsl:if>
      <xsl:if test="name()='initializer'">
	        <xsl:text> = </xsl:text>
		    <xsl:apply-templates select="."/>
	  </xsl:if>
   </xsl:for-each>
   <xsl:text>; </xsl:text>
   <xsl:call-template name="LI_tail"/>
</xsl:template>


<xsl:template name="storage_class_specifier">
   <xsl:if test="name()='auto' or
                 name()='register' or
				 name()='static' or
                 name()='extern' or
                 name()='typedef'">
      <B STYLE="color: olive"><xsl:value-of select="name()"/></B>
      <xsl:text> </xsl:text>
   </xsl:if>
</xsl:template>   


<xsl:template name="type_specifier">
   <xsl:if test="name()='void' or
                 name()='char' or
                 name()='short' or
                 name()='int' or
                 name()='long' or
                 name()='float' or
                 name()='double' or
                 name()='signed' or
                 name()='unsigned'">
      <B STYLE="color: olive"><xsl:value-of select="name()"/></B>
      <xsl:text> </xsl:text>
   </xsl:if>
   <xsl:if test="name()='struct_or_union_specifier' or
                 name()='enum_specifier' or
				 name()='typedef_name'">
	  <xsl:apply-templates select="."/>
	  <xsl:text> </xsl:text>
   </xsl:if>	  
</xsl:template>


<xsl:template name="type_qualifier">
   <xsl:if test="name()='const' or name()='volatile'">
      <B STYLE="color: olive"><xsl:value-of select="name()"/></B>
	  <xsl:text> </xsl:text>
   </xsl:if>
</xsl:template>


<xsl:template name="declaration_specifiers">
   <xsl:call-template name="storage_class_specifier"/>
   <xsl:call-template name="type_specifier"/>
   <xsl:call-template name="type_qualifier"/>
</xsl:template>


<xsl:template match="struct_or_union_specifier">
   <A NAME="{@id}"></A>
   <xsl:if test="not(struct_declaration)">
      <B STYLE="color: orange"><xsl:value-of select="@rhs"/></B>
      <xsl:apply-templates/>
   </xsl:if>
   <xsl:if test="struct_declaration">
      <B STYLE="color: orange"><xsl:value-of select="@rhs"/></B>
	  <xsl:text> </xsl:text>			
	  <xsl:apply-templates select="identifier"/>
	  <xsl:text> { </xsl:text>
	  <xsl:call-template name="LI_tail"/>
   </xsl:if>
   <xsl:for-each select="struct_declaration">
	  <xsl:call-template name="LI_head"/>
	  <xsl:text> &#160;&#160;&#160;&#160;&#160; </xsl:text>
      <xsl:apply-templates select="."/>
      <xsl:text>; </xsl:text>
      <xsl:call-template name="LI_tail"/>
	  <xsl:if test="position()=last()">
	     <xsl:choose>
	        <xsl:when test="../following-sibling::declarator">
		       <xsl:call-template name="LI_head">
			      <xsl:with-param  name="current_lineno">
			         <xsl:value-of
					    select="following::identifier[1]/@lineno"/>
			      </xsl:with-param>
			   </xsl:call-template>
		    </xsl:when>
		    <xsl:otherwise>
			   <xsl:call-template name="LI_head"/>
			</xsl:otherwise>
		 </xsl:choose>
		 <xsl:text> &#160;&#160;&#160;&#160;&#160; </xsl:text>
	     <xsl:text> } </xsl:text>
	  </xsl:if>
   </xsl:for-each>      
</xsl:template>


<xsl:template match="struct_declaration">
   <A NAME="{@id}"></A>
   <xsl:for-each select="*">
      <xsl:call-template name="type_specifier"/>
	  <xsl:call-template name="type_qualifier"/>
      <xsl:if test="@rhs='constant'">
	     <xsl:text>: </xsl:text>
	  </xsl:if>
	  <xsl:if test="declarator or expression">
	     <xsl:apply-templates select="."/>
	  </xsl:if>
   </xsl:for-each>
</xsl:template>


<xsl:template match="enum_specifier">
   <A NAME="{@id}"></A>
   <xsl:if test="@rhs='notag'">
      <xsl:for-each select="*">
	     <xsl:if test="position()=1">
			<B STYLE="color: orange"> enum </B>
	        <xsl:text> { </xsl:text>
			<xsl:call-template name="LI_tail"/>
			<xsl:call-template name="LI_head">
			   <xsl:with-param name="current_lineno">
			      <xsl:value-of select="@lineno"/>
			   </xsl:with-param>
			</xsl:call-template>
			<xsl:text> &#160;&#160;&#160;&#160;&#160; </xsl:text>
			<xsl:apply-templates select="."/>
	     </xsl:if>
		 <xsl:if test="1 &lt; position() and name()='identifier'">
		    <xsl:text>, </xsl:text>
			<xsl:call-template name="LI_tail"/>
			<xsl:call-template name="LI_head">
			   <xsl:with-param name="current_lineno">
			      <xsl:value-of select="@lineno"/>
			   </xsl:with-param>
			</xsl:call-template>
			<xsl:text> &#160;&#160;&#160;&#160;&#160; </xsl:text>
			<xsl:apply-templates select="."/>
		 </xsl:if>
		 <xsl:if test="name()='expression'">
		    <xsl:text> = </xsl:text>
			<xsl:apply-templates select="."/>
		 </xsl:if>
	     <xsl:if test="position()=last()">
			<xsl:call-template name="LI_tail"/>
	        <xsl:choose>
	           <xsl:when test="../following-sibling::declarator">
		          <xsl:call-template name="LI_head">
			         <xsl:with-param  name="current_lineno">
			            <xsl:value-of 
						   select="following::identifier[1]/@lineno"/>
			         </xsl:with-param>
			      </xsl:call-template>
		       </xsl:when>
		       <xsl:otherwise>
			      <xsl:call-template name="LI_head"/>
			   </xsl:otherwise>
		    </xsl:choose>
			<xsl:text> &#160;&#160;&#160;&#160;&#160; </xsl:text>
	        <xsl:text> } </xsl:text>
	     </xsl:if>
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='normal' or @rhs='nobody'">
      <xsl:for-each select="*">
	     <xsl:if test="position()=1">
			<B STYLE="color: orange"> enum </B>
		    <xsl:apply-templates select="identifier"/>
		 </xsl:if>
		 <xsl:if test="position()=2">
		    <xsl:text> { </xsl:text>
			<xsl:call-template name="LI_tail"/>
			<xsl:call-template name="LI_head">
			   <xsl:with-param name="current_lineno">
			      <xsl:value-of select="@lineno"/>
			   </xsl:with-param>
			</xsl:call-template>
			<xsl:text> &#160;&#160;&#160;&#160;&#160; </xsl:text>
			<xsl:apply-templates select="."/>
		 </xsl:if>
		 <xsl:if test="2 &lt; position() and name()='identifier'">
		    <xsl:text>, </xsl:text><BR/>
			<xsl:call-template name="LI_tail"/>
			<xsl:call-template name="set-tab"/>
			<xsl:apply-templates select="."/>
			<xsl:call-template name="LI_head">
			   <xsl:with-param name="current_lineno">
			      <xsl:value-of select="lineno"/>
			   </xsl:with-param>
			</xsl:call-template>
		 </xsl:if>
		 <xsl:if test="name()='expression'">
		    <xsl:text> = </xsl:text>
			<xsl:apply-templates select="."/>
		 </xsl:if>
	     <xsl:if test="position()=last()">
		 	<xsl:call-template name="LI_tail"/>
	        <xsl:choose>
	           <xsl:when test="../following-sibling::declarator">
		          <xsl:call-template name="LI_head">
			         <xsl:with-param  name="current_lineno">
			            <xsl:value-of 
						   select="following::identifier[1]/@lineno"/>
			         </xsl:with-param>
			      </xsl:call-template>
		       </xsl:when>
		       <xsl:otherwise>
			      <xsl:call-template name="LI_head"/>
			   </xsl:otherwise>
		    </xsl:choose>
			<xsl:text> &#160;&#160;&#160;&#160;&#160; </xsl:text>
	        <xsl:text> } </xsl:text>
	     </xsl:if>
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='nobody'">
      <xsl:apply-templates/>
   </xsl:if>
</xsl:template>


<xsl:template match="declarator">
   <A NAME="{@id}"></A>
   <xsl:if test="@rhs='array'">
      <xsl:apply-templates select="declarator"/>
      <xsl:text> [ </xsl:text>
      <xsl:for-each select="expression">
         <xsl:apply-templates select="."/>
      </xsl:for-each>
      <xsl:text> ] </xsl:text>
   </xsl:if>

   <xsl:if test="@rhs='id'">
      <xsl:apply-templates/>
   </xsl:if>

   <xsl:if test="@rhs='paren'">
      <xsl:text> ( </xsl:text>
      <xsl:apply-templates/>
      <xsl:text> ) </xsl:text>
      </xsl:if>

   <xsl:if test="@rhs='pointer' or @rhs='pointer_null'">
      <xsl:apply-templates/>
   </xsl:if>

   <xsl:if test="@rhs='func_new'">
      <xsl:apply-templates select="declarator"/>
      <xsl:for-each select="parameter_declaration">
         <xsl:if test="position()=1">
	        <xsl:text> ( </xsl:text>
		    <xsl:apply-templates select="."/>
   	     </xsl:if>
	     <xsl:if test="1 &lt; position()">
	        <xsl:text>, </xsl:text>
		    <xsl:apply-templates select="."/>
	     </xsl:if>
	     <xsl:if test="position()=last()">
            <xsl:text> ) </xsl:text>
	     </xsl:if>
      </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='func_old'">
      <xsl:apply-templates select="declarator"/>
      <xsl:text>(</xsl:text>
	   <xsl:for-each select="identifier">
         <xsl:if test="position()=1">
	        <xsl:apply-templates select="."/>
	     </xsl:if>
	     <xsl:if test="1 &lt; position()">
	        <xsl:text>, </xsl:text>
			<xsl:apply-templates select="."/>
	     </xsl:if>
      </xsl:for-each>
      <xsl:text>)</xsl:text>
   </xsl:if>
</xsl:template>


<xsl:template match="ellipsis">
   <xsl:text> ... </xsl:text>
</xsl:template>


<xsl:template match="pointer">
   <xsl:text>*</xsl:text>
   <xsl:for-each select="*">
      <xsl:call-template name="type_qualifier"/>	  
   </xsl:for-each>
   <xsl:apply-templates/>
</xsl:template>


<xsl:template match="parameter_declaration">
   <xsl:for-each select="*">
      <xsl:call-template name="declaration_specifiers"/>
	  <xsl:if test="name()='declarator' or
	                name()='abstract_declarator'">
         <xsl:apply-templates select="."/>
	  </xsl:if>				
   </xsl:for-each>
</xsl:template>


<xsl:template match="abstract_declarator">
   <xsl:if test="@rhs='pointer_nil' or @rhs='pointer_pair'">
   <xsl:apply-templates/>
   </xsl:if>
   <xsl:if test="@rhs='paren'">
      <xsl:text> ( </xsl:text>
	  <xsl:apply-templates/>
      <xsl:text> ) </xsl:text>
   </xsl:if>
   <xsl:if test="@rhs='array_nil' or @rhs='array_pair'">
      <xsl:apply-templates select="abstract_declarator"/>
      <xsl:text> [ </xsl:text>
      <xsl:for-each select="expression">
         <xsl:apply-templates select="."/>
      </xsl:for-each>
      <xsl:text> ] </xsl:text>
 </xsl:if>
   <xsl:if test="@rhs='func_nil' or @rhs='func_pair'">
      <xsl:apply-templates select="abstract_declarator"/>
      <xsl:for-each select="parameter_declaration">
         <xsl:if test="position()=1">
	        <xsl:text> ( </xsl:text>
		    <xsl:apply-templates select="."/>
	     </xsl:if>
	     <xsl:if test="1 &lt; position()">
	        <xsl:text>, </xsl:text>
		    <xsl:apply-templates select="."/>
	     </xsl:if>
	     <xsl:if test="position()=last()">
            <xsl:text> ) </xsl:text>
	     </xsl:if>
      </xsl:for-each>
   </xsl:if>
</xsl:template>


<xsl:template match="initializer">
   <xsl:if test="@rhs='exp'">
      <xsl:apply-templates/>
   </xsl:if>
   <xsl:if test="@rhs='list'">
      <xsl:text> { </xsl:text>
	  <xsl:for-each select="*">
		 <xsl:if test="@rhs='exp' or @rhs='list'">
		    <xsl:if test="position()=1">
			   <xsl:apply-templates select="."/>
		    </xsl:if>
			<xsl:if test="1 &lt; position()">
			   <xsl:text>, </xsl:text>
			   <xsl:apply-templates select="."/>
			</xsl:if>
		 </xsl:if>
	  </xsl:for-each>
      <xsl:text> } </xsl:text>
   </xsl:if>
</xsl:template>


<xsl:template match="type_name">
   <xsl:for-each select="*">
      <xsl:call-template name="type_specifier"/>
      <xsl:call-template name="type_qualifier"/>
	  <xsl:if test="name()='abstract_declarator'">
         <xsl:apply-templates select="."/>
	  </xsl:if>
   </xsl:for-each>
</xsl:template>


<xsl:template match="statement">
   <xsl:if test="@rhs='compound'">      
	  <xsl:if test="parent::function_definition">
	     <xsl:text disable-output-escaping="yes"> &lt;LI> </xsl:text>
	     <xsl:text> { </xsl:text>
		 <xsl:call-template name="LI_tail"/>
	  </xsl:if>	 
      <xsl:for-each select="*">
	     <xsl:if test="name()='declaration' or name()='statement'">
			<xsl:if test="@rhs='case'">
			   <xsl:call-template name="LI_head">
			      <xsl:with-param name="current_lineno">
				     <xsl:value-of
					    select="descendant::constant[1]/@lineno"/>
				  </xsl:with-param>
			   </xsl:call-template>
			   <xsl:call-template name="set-tab-"/>
			</xsl:if>
			<xsl:if test="@rhs='default'">
   	           <xsl:call-template name="LI_head"/>			
			   <xsl:call-template name="set-tab-"/>
			</xsl:if>
			<xsl:if test="not(@rhs='case') and not(@rhs='default')">
	           <xsl:call-template name="LI_head"/>			
			   <xsl:call-template name="set-tab"/>
			</xsl:if>
		 </xsl:if>
         <xsl:apply-templates select="."/>
	  </xsl:for-each>
	  <xsl:text disable-output-escaping="yes"> &lt;LI> </xsl:text>
	  <xsl:if test="parent::function_definition">
	     <xsl:call-template name="set-tab-"/>
	  </xsl:if>
	  <xsl:if test="not(parent::function_definition)">
	     <xsl:call-template name="set-tab"/>
	  </xsl:if>
	  <xsl:if test="not(parent::statement/@rhs='do-while')">
         <xsl:text> } </xsl:text>
	  </xsl:if>
	  <xsl:call-template name="LI_tail"/>
   </xsl:if>

   <xsl:if test="@rhs='switch'">
      <xsl:for-each select="*">
	     <xsl:if test="name()='expression'">
            <B STYLE="color: brown">switch </B>
	        <xsl:text> ( </xsl:text>
	        <xsl:apply-templates select="."/>
	        <xsl:text> ){ </xsl:text>						
			<xsl:call-template name="LI_tail"/>
		 </xsl:if>
		 <xsl:if test="name()='statement'">		 
		    <xsl:apply-templates select="."/>
		 </xsl:if>	
	  </xsl:for-each>
   </xsl:if>
   <xsl:if test="@rhs='case'">      
      <xsl:for-each select="*">
	     <xsl:if test="position()=1">
            <B STYLE="color: brown">case </B>
	        <xsl:if test="name()='expression'">                  
	           <xsl:apply-templates select="."/>
	           <xsl:text>: </xsl:text>
	           <xsl:call-template name="LI_tail"/>
	        </xsl:if>
		 </xsl:if>
		 <xsl:if test="name()='statement'">
		    <xsl:call-template name="LI_head"/>
			<xsl:call-template name="set-tab-"/>
		    <xsl:apply-templates select="."/>
		 </xsl:if>			 
	  </xsl:for-each>
   </xsl:if>
   <xsl:if test="@rhs='default'">
      <B STYLE="color: brown">default </B>
	  <xsl:text>: </xsl:text>
	  <xsl:for-each select="*">
	     <xsl:call-template name="LI_head"/>
		 <xsl:call-template name="set-tab-"/>
	     <xsl:apply-templates select="."/>
      </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='if' or
                 @rhs='if-else' or
				 @rhs='while'">
      <xsl:if test="@rhs='if' or @rhs='if-else'">
         <B STYLE="color: brown">if </B>
	  </xsl:if>
	  <xsl:if test="@rhs='while'">				 
         <B STYLE="color: brown"><xsl:value-of select="@rhs"/></B>
	  </xsl:if>
      <xsl:for-each select="*">
		 <xsl:if test="name()='expression'">
		    <xsl:text> ( </xsl:text>
		    <xsl:apply-templates select="."/>
			<xsl:if test="following-sibling::statement[1]/@rhs='compound'">
	        <xsl:text> ){ </xsl:text>
			</xsl:if>
			<xsl:if 
			   test="not(following-sibling::statement[1]/@rhs='compound')">
			<xsl:text> ) </xsl:text>
			</xsl:if>
		 </xsl:if>	
         <xsl:if test="position()=2">
			<xsl:call-template name="LI_tail"/>
			<xsl:if test="not(@rhs='compound')">
	           <xsl:call-template name="LI_head"/>
			   <xsl:call-template name="set-tab+"/>
			</xsl:if>
		    <xsl:apply-templates select="."/>
		 </xsl:if>
		 <xsl:if test="position()=3">		    
		    <xsl:if test="not(@rhs='compound')">
		       <xsl:call-template name="LI_head"/>
			</xsl:if>
			<xsl:if test="@rhs='compound'">
	           <xsl:text disable-output-escaping="yes"> 
			      &lt;LI>
               </xsl:text>
			</xsl:if>
<!-- -->
            <xsl:choose>
            <xsl:when
            test="(parent::statement/preceding-sibling::statement and
            parent::statement/parent::statement/preceding-sibling::statement)
            or  ../../../../function_definition">
<!--
<xsl:text> HOGE0 </xsl:text><BR/>
-->
			   <xsl:call-template name="set-tab-"/>
			</xsl:when>
			<xsl:otherwise>
<!--
<xsl:text> HOGE1 </xsl:text><BR/>
-->
			   <xsl:call-template name="set-tab"/>
			</xsl:otherwise>
			</xsl:choose>

		    <B STYLE="color: brown">else </B>
			<xsl:if test="@rhs='compound'">
			   <xsl:text> { </xsl:text>
			</xsl:if>
			<xsl:if test="not(@rhs='if') and 
			              not(@rhs='if-else') and 
						  not(@rhs='while')">
			   <xsl:call-template name="LI_tail"/>
			</xsl:if>
			<xsl:if test="not(@rhs='compound') and
			              not(@rhs='if') and
						  not(@rhs='if-else') and
						  not(@rhs='while')">
			   <xsl:call-template name="LI_head"/>
			   <xsl:call-template name="set-tab+"/>
			</xsl:if>
			<xsl:apply-templates select="."/>
		 </xsl:if>
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='for'">
      <B STYLE="color: brown"><xsl:value-of select="@rhs"/></B>
	  <xsl:text> ( </xsl:text>
	  <xsl:for-each select="*">
	     <xsl:if test="position()=1 or position()=2">
		    <xsl:apply-templates select="."/>
			<xsl:text>; </xsl:text>
		 </xsl:if>
		 <xsl:if test="position()=3">
		    <xsl:apply-templates select="."/>
			<xsl:if test="following-sibling::statement[1]/@rhs='compound'">
	           <xsl:text> ){ </xsl:text>
			</xsl:if>
			<xsl:if 
			   test="not(following-sibling::statement[1]/@rhs='compound')">
			   <xsl:text> ) </xsl:text>
			</xsl:if>
		 </xsl:if>
		 <xsl:if test="position()=last()">
 			<xsl:call-template name="LI_tail"/>
			<xsl:if test="not(@rhs='compound')">
	           <xsl:call-template name="LI_head"/>
			   <xsl:call-template name="set-tab+"/>
			</xsl:if>
		    <xsl:apply-templates select="."/>
		 </xsl:if>
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='do-while'">
      <xsl:for-each select="*">
	     <xsl:if test="name()='statement'">		    
			<xsl:if test="@rhs='compound'">
			   <B STYLE="color: brown">do </B>
			   <xsl:text> { </xsl:text>
			   <xsl:call-template name="LI_tail"/>
			   <xsl:apply-templates select="."/>
			</xsl:if>
			<xsl:if test="not(@rhs='compound')">
			   <B STYLE="color: brown">do </B>	   
			   <xsl:apply-templates select="."/>
			</xsl:if>
		 </xsl:if>
		 <xsl:if test="name()='expression'">
		    <xsl:call-template name="LI_head"/>
			<xsl:call-template name="set-tab"/>
			<xsl:if test="preceding-sibling::statement/@rhs='compound'">
			   <xsl:text> } </xsl:text><B STYLE="color: brown"> while </B>
			   <xsl:text> ( </xsl:text>
			   <xsl:apply-templates select="."/>
			   <xsl:text> ); </xsl:text>
			   <xsl:call-template name="LI_tail"/>
			</xsl:if>
			<xsl:if test="not(preceding-sibling::statement/@rhs='compound')">
		       <B STYLE="color: brown">while </B>
			   <xsl:text> ( </xsl:text>
			   <xsl:apply-templates select="."/>
			   <xsl:text> ); </xsl:text>
			   <xsl:call-template name="LI_tail"/>
		    </xsl:if>	   
		 </xsl:if>
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='expression_opt'">
      <xsl:apply-templates/>
	  <xsl:text>; </xsl:text>
	  <xsl:call-template name="LI_tail"/>
   </xsl:if>
   <xsl:if test="@rhs='continue' or @rhs='break'">
      <xsl:apply-templates/>
	  <xsl:text>; </xsl:text>
	  <xsl:call-template name="LI_tail"/>
   </xsl:if>
   <xsl:if test="@rhs='return'">
      <B STYLE="color: brown">return </B>
	  <xsl:apply-templates/>
	  <xsl:text>; </xsl:text>
	  <xsl:call-template name="LI_tail"/>
   </xsl:if>
   <xsl:if test="@rhs='goto'">
      <B STYLE="color: brown">goto </B>
	  <xsl:apply-templates/>
	  <xsl:text>; </xsl:text>
	  <xsl:call-template name="LI_tail"/>
   </xsl:if>
   <xsl:if test="@rhs='label'">
      <xsl:apply-templates select="identifier"/>
	  <xsl:text>: </xsl:text>
	  <xsl:for-each select="*">
		 <xsl:apply-templates select="."/>
	  </xsl:for-each>
   </xsl:if>
</xsl:template>


<xsl:template match="continue | break">
   <B STYLE="color: brown"><xsl:value-of select="name(.)"/></B>   
</xsl:template>


<xsl:template match="empty_expression">
   <xsl:text>  </xsl:text>
</xsl:template>


<xsl:template match="expression">
   <xsl:if test="@rhs='funcall'">
      <xsl:for-each select="expression">
	     <xsl:if test="position()=1">
			<xsl:apply-templates select="."/>
    		<xsl:text>( </xsl:text>
		 </xsl:if>
		 <xsl:if test="1 &lt; position() and position() &lt; last()">
		    <xsl:apply-templates select="."/>
			<xsl:text>, </xsl:text>
		 </xsl:if>
		 <xsl:if test="position()=last()">
		    <xsl:apply-templates select="."/>
			<xsl:text> ) </xsl:text>
		 </xsl:if>
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='array'">      
	  <xsl:for-each select="*">
	     <xsl:if test="position()=1">
	        <xsl:apply-templates select="."/>			
		 </xsl:if>
		 <xsl:if test="position()=last()">
		    <xsl:text> [ </xsl:text>
			<xsl:apply-templates select="."/>
			<xsl:text> ] </xsl:text>
		 </xsl:if>	
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='paren'">
      <xsl:text> ( </xsl:text>
	  <xsl:apply-templates/>
      <xsl:text> ) </xsl:text>
   </xsl:if>

   <xsl:if test="@rhs='identifier'">
      <B STYLE="color: blue"><xsl:apply-templates/></B>	 
   </xsl:if>
   <xsl:if test="@rhs='constant'">
      <xsl:apply-templates/>
   </xsl:if>
   <xsl:if test="@rhs='sizeof-type'">
      <xsl:text> sizeof </xsl:text>
	  <xsl:text> ( </xsl:text>
	  <xsl:apply-templates/>
	  <xsl:text> ) </xsl:text>
   </xsl:if>

   <xsl:if test="@rhs='comma'">
      <xsl:call-template name="binary_op">
      	 <xsl:with-param name="real_code_b"> , </xsl:with-param>
      </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_eq'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> = </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_mult'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> *= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_div'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> /= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_mod'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> %= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_add'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> += </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_sub'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> -= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_lshift'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b">
		    <![CDATA[<<=]]>
		 </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_rshift'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> >>= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_band'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> 
		    <![CDATA[&=]]>
         </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_beor'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> ^= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='assign_bior'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> |= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='logical-or'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> || </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='logical-and'">
      <xsl:call-template name="binary_op">
 	     <xsl:with-param name="real_code_b">
		    <![CDATA[&&]]>
         </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='bitwise-inclusive-or'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> | </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='bitwise-exclusive-or'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> ^ </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='bitwise-and'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b">
		    <![CDATA[&]]> 
		 </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='equal'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> == </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='not-equal'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> != </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='less'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b">
		    <![CDATA[<]]> 
		 </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='more'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> > </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='less-equal'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b">
		    <![CDATA[<=]]> 
		 </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='more-equal'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> >= </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='left-shift'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b">
		    <![CDATA[<<]]>
	     </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='right-shift'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> >> </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='add'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> + </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='sub'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> - </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='mult'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> * </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='div'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> / </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='mod'">
      <xsl:call-template name="binary_op">
	     <xsl:with-param name="real_code_b"> % </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>

   <xsl:if test="@rhs='pre-increment'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> ++ </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='pre-decrement'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> -- </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='unary-address'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u">
		    <![CDATA[&]]>
         </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='unary-deref'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> * </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='unary-add'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> + </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='unary-sub'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> - </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='unary-comp'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> ~ </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='unary-neg'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> ! </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='sizeof-exp'">
      <xsl:call-template name="unary_op">
	     <xsl:with-param name="real_code_u"> sizeof </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>

   <xsl:if test="@rhs='dot'">
      <xsl:call-template name="post_op">
	     <xsl:with-param name="real_code_p"> . </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='arrow'">
      <xsl:call-template name="post_op">
	     <xsl:with-param name="real_code_p"> -> </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='post-increment'">
      <xsl:call-template name="post_op">
	     <xsl:with-param name="real_code_p"> ++ </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>
   <xsl:if test="@rhs='post-decrement'">
      <xsl:call-template name="post_op">
	     <xsl:with-param name="real_code_p"> -- </xsl:with-param>
	  </xsl:call-template>
   </xsl:if>

   <xsl:if test="@rhs='conditional'">
      <xsl:for-each select="*">
	     <xsl:if test="position()=1">
		    <xsl:apply-templates select="."/>
		 </xsl:if>
		 <xsl:if test="position()=2">
		    <xsl:text> ? </xsl:text>
			<xsl:apply-templates select="."/>
		 </xsl:if>
		 <xsl:if test="position()=3">
		    <xsl:text> : </xsl:text>
			<xsl:apply-templates select="."/>
		 </xsl:if>
	  </xsl:for-each>
   </xsl:if>

   <xsl:if test="@rhs='cast'">
      <xsl:for-each select="*">
	     <xsl:if test="name()='type_name'">
		    <xsl:text> ( </xsl:text>
			<xsl:apply-templates select="."/>
            <xsl:text> ) </xsl:text>
		 </xsl:if>		 
		 <xsl:if test="name()='expression'">
		    <xsl:apply-templates select="."/>
		 </xsl:if>
	  </xsl:for-each>
   </xsl:if>
</xsl:template>

<xsl:template name="binary_op">
   <xsl:param name="real_code_b"> real-code </xsl:param>
   <xsl:for-each select="*">
      <xsl:if test="position()=1">
	     <xsl:apply-templates select="."/>
	  </xsl:if>
      <xsl:if test="position()=2">
	     <xsl:value-of select='$real_code_b'/>
	     <xsl:text> </xsl:text>
         <xsl:apply-templates select="."/>
	  </xsl:if>	
   </xsl:for-each>
</xsl:template>

<xsl:template name="unary_op">
   <xsl:param name="real_code_u"> real-code </xsl:param>
   <xsl:value-of select='$real_code_u'/>
   <xsl:text> </xsl:text>
   <xsl:apply-templates/>
</xsl:template>

<xsl:template name="post_op">
   <xsl:param name="real_code_p"> real-code </xsl:param>
   <xsl:apply-templates/>
   <xsl:value-of select='$real_code_p'/>
   <xsl:text> </xsl:text>
</xsl:template>

<xsl:template match="identifier">
   <A NAME="{@id}"></A>
   <xsl:choose>
      <xsl:when test="not(@ref='X0') and
	                  (parent::expression or
	                   parent::struct_or_union_specifier
					     [preceding-sibling::*])">
         <A HREF="#{@ref}">
            <xsl:value-of select="normalize-space()"/>
         </A>
      </xsl:when>
      <xsl:otherwise>
         <xsl:value-of select="normalize-space()"/>
      </xsl:otherwise>
   </xsl:choose>	  
</xsl:template>


<xsl:template match="typedef_name">
   <A NAME="{@id}"></A>
   <A HREF="#{@ref}">
      <B STYLE="color: yellowgreen"><xsl:value-of select="."/></B>
   </A>
</xsl:template>


<xsl:template match="constant">
   <A NAME="{@id}"></A>
   <xsl:if test="@rhs='string'">
      <xsl:text>"</xsl:text>
	     <xsl:value-of select="."/>
	  <xsl:text>"</xsl:text>	 
   </xsl:if>
   <xsl:if test="@rhs='int' or @rhs='float'">
      <xsl:value-of select="."/>
   </xsl:if>
   <xsl:if test="@rhs='char'">
      <xsl:choose>
         <xsl:when test="normalize-space()='0'">
	        <xsl:text> '\0' </xsl:text>
	     </xsl:when>	  
         <xsl:when test="normalize-space()='7'">
	        <xsl:text> '\a' </xsl:text>
		 </xsl:when>	  
		 <xsl:when test="normalize-space()='8'">
	        <xsl:text> '\b' </xsl:text>
		 </xsl:when>	  
		 <xsl:when test="normalize-space()='9'">
	        <xsl:text> '\t' </xsl:text>
		 </xsl:when>	  
		 <xsl:when test="normalize-space()='10'">
	        <xsl:text> '\n' </xsl:text>
		 </xsl:when>	  
		 <xsl:when test="normalize-space()='11'">
	        <xsl:text> '\v' </xsl:text>
		 </xsl:when>	  
		 <xsl:when test="normalize-space()='12'">
	        <xsl:text> '\f' </xsl:text>
		 </xsl:when>	  
		 <xsl:when test="normalize-space()='13'">
	        <xsl:text> '\r' </xsl:text>
		 </xsl:when>	  
		 <xsl:when test="normalize-space()='32'">
	        <xsl:text> '\' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='33'">
	        <xsl:text> '!' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='34'">
	        <xsl:text> '\"' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='35'">
	        <xsl:text> '#' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='36'">
	        <xsl:text> '$' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='37'">
	        <xsl:text> '%' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='38'">
	        <xsl:text> '&amp;' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='39'">
	        <xsl:text> '\'' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='40'">
	        <xsl:text> '(' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='41'">
	        <xsl:text> ')' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='42'">
	        <xsl:text> '*' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='43'">
	        <xsl:text> '+' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='44'">
	        <xsl:text> ',' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='45'">
	        <xsl:text> '-' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='46'">
	        <xsl:text> '.' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='47'">
	        <xsl:text> '/' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='48'">
	        <xsl:text> '0' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='49'">
	        <xsl:text> '1' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='50'">
	        <xsl:text> '2' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='51'">
	        <xsl:text> '3' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='52'">
	        <xsl:text> '4' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='53'">
	        <xsl:text> '5' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='54'">
	        <xsl:text> '6' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='55'">
	        <xsl:text> '7' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='56'">
	        <xsl:text> '8' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='57'">
	        <xsl:text> '9' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='58'">
	        <xsl:text> ':' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='59'">
	        <xsl:text> ';' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='60'">
	        <xsl:text> <![CDATA['<']]> </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='61'">
	        <xsl:text> '=' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='62'">
	        <xsl:text> '>' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='63'">
	        <xsl:text> '\?' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='64'">
	        <xsl:text> '@' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='65'">
	        <xsl:text> 'A' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='66'">
	        <xsl:text> 'B' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='67'">
	        <xsl:text> 'C' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='68'">
	        <xsl:text> 'D' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='69'">
	        <xsl:text> 'E' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='70'">
	        <xsl:text> 'F' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='71'">
	        <xsl:text> 'G' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='72'">
	        <xsl:text> 'H' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='73'">
	        <xsl:text> 'I' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='74'">
	        <xsl:text> 'J' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='75'">
	        <xsl:text> 'K' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='76'">
	        <xsl:text> 'L' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='77'">
	        <xsl:text> 'M' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='78'">
	        <xsl:text> 'N' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='79'">
	        <xsl:text> 'O' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='80'">
	        <xsl:text> 'P' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='81'">
	        <xsl:text> 'Q' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='82'">
	        <xsl:text> 'R' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='83'">
	        <xsl:text> 'S' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='84'">
	        <xsl:text> 'T' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='85'">
	        <xsl:text> 'U' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='86'">
	        <xsl:text> 'V' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='87'">
	        <xsl:text> 'W' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='88'">
	        <xsl:text> 'X' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='89'">
	        <xsl:text> 'Y' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='90'">
	        <xsl:text> 'Z' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='91'">
	        <xsl:text> '[' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='92'">
	        <xsl:text> '\\' </xsl:text>
 		 </xsl:when>
		 <xsl:when test="normalize-space()='93'">
	        <xsl:text> ']' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='94'">
	        <xsl:text> '^' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='95'">
	        <xsl:text> '_' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='96'">
	        <xsl:text> '`' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='97'">
	        <xsl:text> 'a' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='98'">
	        <xsl:text> 'b' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='99'">
	        <xsl:text> 'c' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='100'">
	        <xsl:text> 'd' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='101'">
	        <xsl:text> 'e' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='102'">
	        <xsl:text> 'f' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='103'">
	        <xsl:text> 'g' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='104'">
	        <xsl:text> 'h' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='105'">
	        <xsl:text> 'i' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='106'">
	        <xsl:text> 'j' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='107'">
	        <xsl:text> 'k' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='108'">
	        <xsl:text> 'l' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='109'">
	        <xsl:text> 'm' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='110'">
	        <xsl:text> 'n' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='111'">
	        <xsl:text> 'o' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='112'">
	        <xsl:text> 'p' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='113'">
	        <xsl:text> 'q' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='114'">
	        <xsl:text> 'r' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='115'">
	        <xsl:text> 's' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='116'">
	        <xsl:text> 't' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='117'">
	        <xsl:text> 'u' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='118'">
	        <xsl:text> 'v' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='119'">
	        <xsl:text> 'w' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='120'">
	        <xsl:text> 'x' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='121'">
	        <xsl:text> 'y' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='122'">
	        <xsl:text> 'z' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='123'">
	        <xsl:text> '{' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='124'">
	        <xsl:text> '|' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='125'">
	        <xsl:text> '}' </xsl:text>
		 </xsl:when>
		 <xsl:when test="normalize-space()='126'">
	        <xsl:text> '~' </xsl:text>
		 </xsl:when>
		 <xsl:otherwise>
		    <xsl:value-of select="."/>
		 </xsl:otherwise>	  
      </xsl:choose>
   </xsl:if>
</xsl:template>


<!--
<xsl:template match="symbol">
</xsl:template>

-->

</xsl:stylesheet>