import org.w3c.dom.Node;

class NodeIdLink{
  public String sData;
  public String ident;
  public Node nData;
  public NodeIdLink next;
  public NodeIdLink previous;
  private GetAttr attr;

  public NodeIdLink(Node node){
    attr = new GetAttr(node);
    sData = attr.getAttributeByNode(node, "id");
    nData = node;
    if (node.getNodeName().equals("identifier")){
      ident = node.getFirstChild().getNodeValue().trim();
    }
    else {
      ident = new String();
    }
  }

  public String displayLink(){
    return (sData);
  }
}


class NodeIdList{
  private NodeIdLink first;
  private NodeIdLink last;
  private GetAttr attr;

  public NodeIdList(){
    first = null;
    last = null;
    attr = new GetAttr();
  }

  public boolean isEmpty(){
    return (first == null);
  }

  public void insertFirst(Node node){
    NodeIdLink newLink = new NodeIdLink(node);
    if (isEmpty()) {
      last = newLink;
    }
    else {
      first.previous = newLink;
    }
    newLink.next = first;
    first = newLink;
  }

  public void insertLast(Node node){
    NodeIdLink newLink = new NodeIdLink(node);
    if (isEmpty()){
      first = newLink;
    }
    else {
      last.next = newLink;
      newLink.previous = last;
    }
    last = newLink;
  }

  public NodeIdLink deleteFirst(){
    NodeIdLink temp = first;
    if (first.next == null){
      last = null;
    }
    else {
      first.next.previous = null;
    }    
    first = first.next;
    return temp;
  }

  public NodeIdLink deleteLast(){
    NodeIdLink temp = last;
    if (first.next == null){
      first = null;
    }
    else {
      last.previous.next = null;
    }
    last = last.previous;
    return temp;
  }

  public NodeIdLink peek(){
    return first;
  }
  
  public NodeIdLink tail(){
    return last;
  }

  public NodeIdLink referByLink(NodeIdLink key){
    NodeIdLink current = first;
    while(!(current.sData.equals(key.sData))){
      if (current.next == null){
	return null;
      }
      else {
	current = current.next;
      }
    }
    return current;
  }

  public NodeIdLink referById(String key){
    NodeIdLink current = first;
    while(!(current.sData.equals(key))){
      if (current.next == null){
	return null;
      }
      else {
	current = current.next;
      }
    }
    return current;
  }

  public NodeIdLink referByNode(Node node){
    String key = attr.getAttributeByNode(node, "id");
    NodeIdLink current = first;
    while(!(current.sData.equals(key))){
      if (current.next == null){
	return null;
      }
      else {
	  current = current.next;
      }
    }
    return current;
  }

  public boolean find(String key){
    NodeIdLink current = first;
    while(!(current.sData.equals(key))){
      if (current.next == null){
	return false;
      }
      else {
	current = current.next;
      }
    }
    return true;
  }

  public Node takeNode(String key){
    NodeIdLink current = first;
    while(!(current.sData.equals(key))){
      if (current.next == null){
	return null;
      }
      else {
	current = current.next;
      }
    } 
    return current.nData;
  }

  public NodeIdLink delete(String key){
    NodeIdLink current = first;
    NodeIdLink previous = first;
    while(!(current.sData.equals(key))){
      current = current.next;
      if (current == null){
	return null;
      }
    }
    if (current == first){
      first = current.next;
    }
    else {
      current.previous.next = current.next;
    }
    if (current == last){
      last = current.previous;
    }
    else {
      current.next.previous = current.previous;
    }
    return current;
  }

  public void diffNodeTail(NodeIdLink link){ 
    if (isEmpty()){
      insertLast(link.nData);
    }
    else if (!(find(link.sData))){
      insertLast(link.nData);
    }
  }

  public void diffNameFirst(Node node){
    NodeIdLink link = new NodeIdLink(node);
    if (!(link.ident.equals(""))){
      if (isEmpty()){
	insertFirst(node);
      }
      else if (!(seekName(link.ident))){
	insertFirst(node);
      }
    }
  }

  public boolean seekName(String key){
    NodeIdLink current = first;
    while(!(current.ident.equals(key))){
      if (current.next == null){
	return false;
      }
      else {
	current = current.next;
      }
    }
    return true;
  }

  public NodeIdLink deleteName(String key){
    NodeIdLink current = first;
    NodeIdLink previous = first;
    while(!(current.ident.equals(key))){
      current = current.next;
      if (current == null){
	return null;
      }
    }
    if (current == first){
      first = current.next;
    }
    else {
      current.previous.next = current.next;
    }
    if (current == last){
      last = current.previous;
    }
    else {
      current.next.previous = current.previous;
    }
    return current;
  }

  public void displayList(){
    System.out.print("List (first-->last): ");
    NodeIdLink current = first;
    while(current != null){
      System.out.print(current.displayLink()+" ");
      current = current.next;
    }
    System.out.println("");
  }

  public void displayName(){
    System.out.print("ListName (first-->last): ");
    NodeIdLink current = first;
    while(current != null){
      System.out.print(current.ident+" ");
      current = current.next;
    }
    System.out.println("");
  }

}
