/*
    XCI: eXperimental C Interpreter
    -- Copyright (c) 2000-2001, gondow@jais.ac.jp
    $Id: AST.c,v 1.1.1.1 2002/06/06 01:41:56 gondow Exp $
*/ 

#include <stdio.h>
#include <assert.h>
#include <stdarg.h>
#include <string.h>

#include "AST.h"
#include "xci.h"
#include "symbol.h"


/*
  2 assumptions:
  (1) create_AST_ is called via call_AST defined in xci.h.
  (2) variable-length arguments are all typed with non-NULL struct AST *,
      and terminated with (struct AST *) NULL;
      this is not true for IS_AST_LEAF () is true.
 */

struct AST *
create_AST_ (enum AST_Type ast_type, ...)
{
    va_list      argp;
    int          argp_len, i;
    struct AST 	 *ast;
    char    	 *string;

    /* counting the length of varargs */
    if (!is_AST_Type_LEAF (ast_type)) {
	va_start (argp, ast_type);
	argp_len=0;
	while (va_arg (argp, struct AST*) != NULL) {
	    argp_len++;
	}
	va_end (argp);
    }

    /* create and initialize AST members */
    ast = emalloc (sizeof (struct AST));

    ast->ast_type  = ast_type;
    if (is_AST_Type_LEAF (ast_type)) {
	ast->num_child = 0;
    } else {
	ast->num_child = argp_len;
    }

    ast->attr         = NULL;
    ast->type         = NULL;
    ast->prom_type    = NULL;
    ast->filename     = filename;
    ast->inc_filename = inc_filename;
    ast->lineno       = lineno;

    va_start (argp, ast_type);
    switch (ast_type) {
    default:
    	ast->u.child = emalloc (sizeof(struct AST *) * argp_len);
	for (i=0; i<argp_len; i++) {
	    struct AST *child = va_arg (argp, struct AST *);
	    ast->u.child [i] = child;
	    if (child != NULL) {
		child->parent = ast;
		child->nth    = i;
	    }
    	}
	break;
    case AST_IDENTIFIER:
    case AST_TYPEDEF_NAME:
    case AST_ENUMERATION_CONSTANT:
    case AST_STRING:
	string = va_arg (argp, char *);
	ast->u.string = emalloc (strlen (string) + 1);
    	strcpy (ast->u.string, string);
	break;
    case AST_INTEGER_CONSTANT:
	ast->u.int_ul = va_arg (argp, unsigned long);
    	ast->status   = va_arg (argp, unsigned int);
	break;
    case AST_CHARACTER_CONSTANT:
	ast->u.char_i = va_arg (argp, int);
	break;
    case AST_FLOATING_CONSTANT:
	ast->u.float_ld = emalloc (sizeof (long double));
	*ast->u.float_ld = va_arg (argp, long double);
	ast->status      = va_arg (argp, unsigned int);
	break;
    }
    va_end (argp);

    return ast;
}

int
is_AST_Type_LEAF (enum AST_Type ast_type)
{
    switch (ast_type) {
    case AST_IDENTIFIER:
    case AST_TYPEDEF_NAME:
    case AST_ENUMERATION_CONSTANT:
    case AST_STRING:
    case AST_INTEGER_CONSTANT:
    case AST_CHARACTER_CONSTANT:
    case AST_FLOATING_CONSTANT:
	return 1;
    default:
	return 0;
    }
}

int
is_AST_LEAF (struct AST const *ast)
{
    return is_AST_Type_LEAF (ast->ast_type);
}

int
is_AST_declaration_specifiers (struct AST const *ast)
{
    switch (ast->ast_type) {
    case AST_declaration_specifers_storage:
    case AST_declaration_specifers_storage_pair:
    case AST_declaration_specifers_spec:
    case AST_declaration_specifers_spec_pair:
    case AST_declaration_specifers_qual:
    case AST_declaration_specifers_qual_pair:
	return 1;
    default:
	return 0;
    }
}

int
is_AST_specifier_qualifier_list (struct AST const *ast)
{
    switch (ast->ast_type) {
    case AST_specifier_qualifier_list_spec:
    case AST_specifier_qualifier_list_spec_pair:
    case AST_specifier_qualifier_list_qual:
    case AST_specifier_qualifier_list_qual_pair:
	return 1;
    default:
	return 0;
    }
}

int
is_AST_expression (struct AST const *ast)
{
    /* I hate this kind of awkward definition. */
    switch (ast->ast_type) {
    case AST_constant_expression_direct:
    case AST_expression_direct:
    case AST_expression_comma:
    case AST_assignment_expression_direct:
    case AST_assignment_expression_assign:
    case AST_conditional_expression_direct:
    case AST_conditional_expression_cond:
    case AST_logical_or_expression_direct:
    case AST_logical_or_expression_lor:
    case AST_logical_and_expression_direct:
    case AST_logical_and_expression_land:
    case AST_inclusive_or_expression_direct:
    case AST_inclusive_or_expression_bior:
    case AST_exclusive_or_expression_direct:
    case AST_exclusive_or_expression_beor:
    case AST_and_expression_direct:
    case AST_and_expression_band:
    case AST_equality_expression_direct:
    case AST_equality_expression_eq:
    case AST_equality_expression_neq:
    case AST_relational_expression_direct:
    case AST_relational_expression_less:
    case AST_relational_expression_more:
    case AST_relational_expression_lesseq:
    case AST_relational_expression_moreeq:
    case AST_shift_expression_direct:
    case AST_shift_expression_lshift:
    case AST_shift_expression_rshift:
    case AST_additive_expression_direct:
    case AST_additive_expression_add:
    case AST_additive_expression_sub:
    case AST_multiplicative_expression_direct:
    case AST_multiplicative_expression_mult:
    case AST_multiplicative_expression_div:
    case AST_multiplicative_expression_mod:
    case AST_cast_expression_direct:
    case AST_cast_expression_cast:
    case AST_unary_expression_direct:
    case AST_unary_expression_inc:
    case AST_unary_expression_dec:
    case AST_unary_expression_unary_op:
    case AST_unary_expression_sizeof_exp:
    case AST_unary_expression_sizeof_type:
    case AST_postfix_expression_direct:
    case AST_postfix_expression_array:
    case AST_postfix_expression_funcall:
    case AST_postfix_expression_funcall_noarg:
    case AST_postfix_expression_dot:
    case AST_postfix_expression_arrow:
    case AST_postfix_expression_inc:
    case AST_postfix_expression_dec:
    case AST_primary_expression_ID:
    case AST_primary_expression_constant:
    case AST_primary_expression_STRING:
    case AST_primary_expression_paren:
    case AST_argument_expression_list_single:
    case AST_argument_expression_list_pair:
    case AST_constant_INT:
    case AST_constant_CHAR:
    case AST_constant_FLOAT:
    case AST_constant_ENUM:
    case AST_STRING:
    case AST_IDENTIFIER:
    case AST_ENUMERATION_CONSTANT:
    case AST_INTEGER_CONSTANT:
    case AST_CHARACTER_CONSTANT:
    case AST_FLOATING_CONSTANT:
	return 1;

    case AST_translation_unit_single:
    case AST_translation_unit_pair:
    case AST_external_declaration_func:
    case AST_external_declaration_dec:
    case AST_function_definition_spec:
    case AST_function_definition_nospec:
    case AST_declaration_list_opt_null:
    case AST_declaration_list_opt_direct:
    case AST_declaration_direct:
    case AST_init_declarator_list_opt_null:
    case AST_init_declarator_list_opt_direct:
    case AST_declaration_list_single:
    case AST_declaration_list_pair:
    case AST_declaration_specifers_storage:
    case AST_declaration_specifers_storage_pair:
    case AST_declaration_specifers_spec:
    case AST_declaration_specifers_spec_pair:
    case AST_declaration_specifers_qual:
    case AST_declaration_specifers_qual_pair:
    case AST_storage_class_specifier_auto:
    case AST_storage_class_specifier_register:
    case AST_storage_class_specifier_static:
    case AST_storage_class_specifier_extern:
    case AST_storage_class_specifier_typedef:
    case AST_type_specifier_void:
    case AST_type_specifier_char:
    case AST_type_specifier_short:
    case AST_type_specifier_int:
    case AST_type_specifier_long:
    case AST_type_specifier_float:
    case AST_type_specifier_double:
    case AST_type_specifier_signed:
    case AST_type_specifier_unsigned:
    case AST_type_specifier_struct_or_union:
    case AST_type_specifier_enum:
    case AST_type_specifier_TYPEDEF_NAME:
    case AST_type_qualifier_const:
    case AST_type_qualifier_volatile:
    case AST_struct_or_union_specifier_notag:
    case AST_struct_or_union_specifier_tag:
    case AST_struct_or_union_specifier_incomplete:
    case AST_struct_or_union_struct:
    case AST_struct_or_union_union:
    case AST_struct_declaration_list_single:
    case AST_struct_declaration_list_pair:
    case AST_init_declarator_single:
    case AST_init_declarator_pair:
    case AST_init_declarator_dec:
    case AST_init_declarator_dec_init:
    case AST_struct_declaration_direct:
    case AST_specifier_qualifier_list_spec:
    case AST_specifier_qualifier_list_spec_pair:
    case AST_specifier_qualifier_list_qual:
    case AST_specifier_qualifier_list_qual_pair:
    case AST_struct_declarator_list_single:
    case AST_struct_declarator_list_pair:
    case AST_struct_declarator_direct:
    case AST_struct_declarator_bitfield_noname:
    case AST_struct_declarator_bitfield:
    case AST_enum_specifier_untag:
    case AST_enum_specifier_tag:
    case AST_enum_specifier_dec:
    case AST_enumerator_list_single:
    case AST_enumerator_list_pair:
    case AST_enumerator_id:
    case AST_enumerator_id_init:
    case AST_declarator_direct:
    case AST_pointer_opt_null:
    case AST_pointer_opt_direct:
    case AST_direct_declarator_id:
    case AST_direct_declarator_paren:
    case AST_direct_declarator_array:
    case AST_direct_declarator_func_new:
    case AST_direct_declarator_func_old:
    case AST_function_declarator_direct:
    case AST_direct_function_declarator_func_new:
    case AST_direct_function_declarator_func_old:
    case AST_constant_expression_opt_null:
    case AST_constant_expression_opt_direct:
    case AST_identifier_list_opt_null:
    case AST_identifier_list_opt_direct:
    case AST_pointer_single:
    case AST_pointer_pair:
    case AST_type_qualifier_list_opt_nil:
    case AST_type_qualifier_list_opt_direct:
    case AST_type_qualifier_list_single:
    case AST_type_qualifier_list_pair:
    case AST_parameter_type_list_direct:
    case AST_parameter_type_list_ellipsis:
    case AST_parameter_list_single:
    case AST_parameter_list_pair:
    case AST_parameter_declaration_dec:
    case AST_parameter_declaration_abs:
    case AST_abstract_declarator_opt_null:
    case AST_abstract_declarator_opt_direct:
    case AST_identifier_list_single:
    case AST_identifier_list_pair:
    case AST_initializer_assign:
    case AST_initializer_list:
    case AST_initializer_list_comma:
    case AST_initializer_list_single:
    case AST_initializer_list_pair:
    case AST_type_name_direct:
    case AST_abstract_declarator_pointer:
    case AST_abstract_declarator_abs:
    case AST_direct_abstract_declarator_paren:
    case AST_direct_abstract_declarator_array_nil:
    case AST_direct_abstract_declarator_array_pair:
    case AST_direct_abstract_declarator_func_nil:
    case AST_direct_abstract_declarator_func_pair:
    case AST_parameter_type_list_opt_null:
    case AST_parameter_type_list_opt_direct:
    case AST_statement_labeled:
    case AST_statement_expression:
    case AST_statement_compound:
    case AST_statement_selection:
    case AST_statement_iteration:
    case AST_statement_jump:
    case AST_labeled_statement_label:
    case AST_labeled_statement_case:
    case AST_labeled_statement_default:
    case AST_expression_statement_direct:
    case AST_expression_opt_null:
    case AST_expression_opt_direct:
    case AST_compound_statement_decn:
    case AST_compound_statement_nodecn:
    case AST_statement_list_opt_null:
    case AST_statement_list_opt_direct:
    case AST_statement_list_single:
    case AST_statement_list_pair:
    case AST_selection_statement_if:
    case AST_selection_statement_if_else:
    case AST_selection_statement_switch:
    case AST_iteration_statement_while:
    case AST_iteration_statement_do_while:
    case AST_iteration_statement_for:
    case AST_jump_statement_goto:
    case AST_jump_statement_continue:
    case AST_jump_statement_break:
    case AST_jump_statement_return:
    case AST_assignment_operator_eq:
    case AST_assignment_operator_mult:
    case AST_assignment_operator_div:
    case AST_assignment_operator_mod:
    case AST_assignment_operator_add:
    case AST_assignment_operator_sub:
    case AST_assignment_operator_lshift:
    case AST_assignment_operator_rshift:
    case AST_assignment_operator_band:
    case AST_assignment_operator_beor:
    case AST_assignment_operator_bior:
    case AST_unary_operator_addr:
    case AST_unary_operator_deref:
    case AST_unary_operator_add:
    case AST_unary_operator_sub:
    case AST_unary_operator_comp:
    case AST_unary_operator_neg:
    case AST_TYPEDEF_NAME:
	return 0;

    default:
	assert (0);
	return 0;
    }
}

int
is_AST_break_exit (struct AST const *ast)
{
    switch (ast->ast_type) {
    case AST_selection_statement_switch:
    case AST_iteration_statement_do_while:
    case AST_iteration_statement_while:
    case AST_iteration_statement_for:
	return 1;
    default:
	return 0;
    }
}

int
is_AST_continue_exit (struct AST const *ast)
{
    switch (ast->ast_type) {
    case AST_iteration_statement_do_while:
    case AST_iteration_statement_while:
    case AST_iteration_statement_for:
	return 1;
    default:
	return 0;
    }
}

int
is_AST_function_def (struct AST const *ast)
{
    switch (ast->ast_type) {
    case AST_function_definition_spec:
    case AST_function_definition_nospec:
	return 1;
    default:
	return 0;
    }
}

int
is_AST_compound_statement (struct AST const *ast)
{
    switch (ast->ast_type) {
    case AST_compound_statement_decn:
    case AST_compound_statement_nodecn:
	return 1;
    default:
	return 0;
    }
}

char *
asttype2str (enum AST_Type ast_type)
{
#if defined(AST_CASE)
    assert (0);
#endif
#define AST_CASE(ast_type)     case ast_type: return #ast_type;
    switch (ast_type) {
    default:
	assert (0);
	return NULL;

    AST_CASE (AST_translation_unit_single)
    AST_CASE (AST_translation_unit_pair)
    AST_CASE (AST_external_declaration_func)
    AST_CASE (AST_external_declaration_dec)
    AST_CASE (AST_function_definition_spec)
    AST_CASE (AST_function_definition_nospec)
    AST_CASE (AST_declaration_list_opt_null)
    AST_CASE (AST_declaration_list_opt_direct)
    AST_CASE (AST_declaration_direct)
    AST_CASE (AST_init_declarator_list_opt_null)
    AST_CASE (AST_init_declarator_list_opt_direct)
    AST_CASE (AST_declaration_list_single)
    AST_CASE (AST_declaration_list_pair)
    AST_CASE (AST_declaration_specifers_storage)
    AST_CASE (AST_declaration_specifers_storage_pair)
    AST_CASE (AST_declaration_specifers_spec)
    AST_CASE (AST_declaration_specifers_spec_pair)
    AST_CASE (AST_declaration_specifers_qual)
    AST_CASE (AST_declaration_specifers_qual_pair)
    AST_CASE (AST_storage_class_specifier_auto)
    AST_CASE (AST_storage_class_specifier_register)
    AST_CASE (AST_storage_class_specifier_static)
    AST_CASE (AST_storage_class_specifier_extern)
    AST_CASE (AST_storage_class_specifier_typedef)
    AST_CASE (AST_type_specifier_void)
    AST_CASE (AST_type_specifier_char)
    AST_CASE (AST_type_specifier_short)
    AST_CASE (AST_type_specifier_int)
    AST_CASE (AST_type_specifier_long)
    AST_CASE (AST_type_specifier_float)
    AST_CASE (AST_type_specifier_double)
    AST_CASE (AST_type_specifier_signed)
    AST_CASE (AST_type_specifier_unsigned)
    AST_CASE (AST_type_specifier_struct_or_union)
    AST_CASE (AST_type_specifier_enum)
    AST_CASE (AST_type_specifier_TYPEDEF_NAME)
    AST_CASE (AST_type_qualifier_const)
    AST_CASE (AST_type_qualifier_volatile)
    AST_CASE (AST_struct_or_union_specifier_notag)
    AST_CASE (AST_struct_or_union_specifier_tag)
    AST_CASE (AST_struct_or_union_specifier_incomplete)
    AST_CASE (AST_struct_or_union_struct)
    AST_CASE (AST_struct_or_union_union)
    AST_CASE (AST_struct_declaration_list_single)
    AST_CASE (AST_struct_declaration_list_pair)
    AST_CASE (AST_init_declarator_single)
    AST_CASE (AST_init_declarator_pair)
    AST_CASE (AST_init_declarator_dec)
    AST_CASE (AST_init_declarator_dec_init)
    AST_CASE (AST_struct_declaration_direct)
    AST_CASE (AST_specifier_qualifier_list_spec)
    AST_CASE (AST_specifier_qualifier_list_spec_pair)
    AST_CASE (AST_specifier_qualifier_list_qual)
    AST_CASE (AST_specifier_qualifier_list_qual_pair)
    AST_CASE (AST_struct_declarator_list_single)
    AST_CASE (AST_struct_declarator_list_pair)
    AST_CASE (AST_struct_declarator_direct)
    AST_CASE (AST_struct_declarator_bitfield_noname)
    AST_CASE (AST_struct_declarator_bitfield)
    AST_CASE (AST_enum_specifier_untag)
    AST_CASE (AST_enum_specifier_tag)
    AST_CASE (AST_enum_specifier_dec)
    AST_CASE (AST_enumerator_list_single)
    AST_CASE (AST_enumerator_list_pair)
    AST_CASE (AST_enumerator_id)
    AST_CASE (AST_enumerator_id_init)
    AST_CASE (AST_declarator_direct)
    AST_CASE (AST_pointer_opt_null)
    AST_CASE (AST_pointer_opt_direct)
    AST_CASE (AST_direct_declarator_id)
    AST_CASE (AST_direct_declarator_paren)
    AST_CASE (AST_direct_declarator_array)
    AST_CASE (AST_direct_declarator_func_new)
    AST_CASE (AST_direct_declarator_func_old)
    AST_CASE (AST_function_declarator_direct)
    AST_CASE (AST_direct_function_declarator_func_new)
    AST_CASE (AST_direct_function_declarator_func_old)
    AST_CASE (AST_constant_expression_opt_null)
    AST_CASE (AST_constant_expression_opt_direct)
    AST_CASE (AST_identifier_list_opt_null)
    AST_CASE (AST_identifier_list_opt_direct)
    AST_CASE (AST_pointer_single)
    AST_CASE (AST_pointer_pair)
    AST_CASE (AST_type_qualifier_list_opt_nil)
    AST_CASE (AST_type_qualifier_list_opt_direct)
    AST_CASE (AST_type_qualifier_list_single)
    AST_CASE (AST_type_qualifier_list_pair)
    AST_CASE (AST_parameter_type_list_direct)
    AST_CASE (AST_parameter_type_list_ellipsis)
    AST_CASE (AST_parameter_list_single)
    AST_CASE (AST_parameter_list_pair)
    AST_CASE (AST_parameter_declaration_dec)
    AST_CASE (AST_parameter_declaration_abs)
    AST_CASE (AST_abstract_declarator_opt_null)
    AST_CASE (AST_abstract_declarator_opt_direct)
    AST_CASE (AST_identifier_list_single)
    AST_CASE (AST_identifier_list_pair)
    AST_CASE (AST_initializer_assign)
    AST_CASE (AST_initializer_list)
    AST_CASE (AST_initializer_list_comma)
    AST_CASE (AST_initializer_list_single)
    AST_CASE (AST_initializer_list_pair)
    AST_CASE (AST_type_name_direct)
    AST_CASE (AST_abstract_declarator_pointer)
    AST_CASE (AST_abstract_declarator_abs)
    AST_CASE (AST_direct_abstract_declarator_paren)
    AST_CASE (AST_direct_abstract_declarator_array_nil)
    AST_CASE (AST_direct_abstract_declarator_array_pair)
    AST_CASE (AST_direct_abstract_declarator_func_nil)
    AST_CASE (AST_direct_abstract_declarator_func_pair)
    AST_CASE (AST_parameter_type_list_opt_null)
    AST_CASE (AST_parameter_type_list_opt_direct)
    AST_CASE (AST_statement_labeled)
    AST_CASE (AST_statement_expression)
    AST_CASE (AST_statement_compound)
    AST_CASE (AST_statement_selection)
    AST_CASE (AST_statement_iteration)
    AST_CASE (AST_statement_jump)
    AST_CASE (AST_labeled_statement_label)
    AST_CASE (AST_labeled_statement_case)
    AST_CASE (AST_labeled_statement_default)
    AST_CASE (AST_expression_statement_direct)
    AST_CASE (AST_expression_opt_null)
    AST_CASE (AST_expression_opt_direct)
    AST_CASE (AST_compound_statement_decn)
    AST_CASE (AST_compound_statement_nodecn)
    AST_CASE (AST_statement_list_opt_null)
    AST_CASE (AST_statement_list_opt_direct)
    AST_CASE (AST_statement_list_single)
    AST_CASE (AST_statement_list_pair)
    AST_CASE (AST_selection_statement_if)
    AST_CASE (AST_selection_statement_if_else)
    AST_CASE (AST_selection_statement_switch)
    AST_CASE (AST_iteration_statement_while)
    AST_CASE (AST_iteration_statement_do_while)
    AST_CASE (AST_iteration_statement_for)
    AST_CASE (AST_jump_statement_goto)
    AST_CASE (AST_jump_statement_continue)
    AST_CASE (AST_jump_statement_break)
    AST_CASE (AST_jump_statement_return)
    AST_CASE (AST_expression_direct)
    AST_CASE (AST_expression_comma)
    AST_CASE (AST_assignment_expression_direct)
    AST_CASE (AST_assignment_expression_assign)
    AST_CASE (AST_assignment_operator_eq)
    AST_CASE (AST_assignment_operator_mult)
    AST_CASE (AST_assignment_operator_div)
    AST_CASE (AST_assignment_operator_mod)
    AST_CASE (AST_assignment_operator_add)
    AST_CASE (AST_assignment_operator_sub)
    AST_CASE (AST_assignment_operator_lshift)
    AST_CASE (AST_assignment_operator_rshift)
    AST_CASE (AST_assignment_operator_band)
    AST_CASE (AST_assignment_operator_beor)
    AST_CASE (AST_assignment_operator_bior)
    AST_CASE (AST_conditional_expression_direct)
    AST_CASE (AST_conditional_expression_cond)
    AST_CASE (AST_constant_expression_direct)
    AST_CASE (AST_logical_or_expression_direct)
    AST_CASE (AST_logical_or_expression_lor)
    AST_CASE (AST_logical_and_expression_direct)
    AST_CASE (AST_logical_and_expression_land)
    AST_CASE (AST_inclusive_or_expression_direct)
    AST_CASE (AST_inclusive_or_expression_bior)
    AST_CASE (AST_exclusive_or_expression_direct)
    AST_CASE (AST_exclusive_or_expression_beor)
    AST_CASE (AST_and_expression_direct)
    AST_CASE (AST_and_expression_band)
    AST_CASE (AST_equality_expression_direct)
    AST_CASE (AST_equality_expression_eq)
    AST_CASE (AST_equality_expression_neq)
    AST_CASE (AST_relational_expression_direct)
    AST_CASE (AST_relational_expression_less)
    AST_CASE (AST_relational_expression_more)
    AST_CASE (AST_relational_expression_lesseq)
    AST_CASE (AST_relational_expression_moreeq)
    AST_CASE (AST_shift_expression_direct)
    AST_CASE (AST_shift_expression_lshift)
    AST_CASE (AST_shift_expression_rshift)
    AST_CASE (AST_additive_expression_direct)
    AST_CASE (AST_additive_expression_add)
    AST_CASE (AST_additive_expression_sub)
    AST_CASE (AST_multiplicative_expression_direct)
    AST_CASE (AST_multiplicative_expression_mult)
    AST_CASE (AST_multiplicative_expression_div)
    AST_CASE (AST_multiplicative_expression_mod)
    AST_CASE (AST_cast_expression_direct)
    AST_CASE (AST_cast_expression_cast)
    AST_CASE (AST_unary_expression_direct)
    AST_CASE (AST_unary_expression_inc)
    AST_CASE (AST_unary_expression_dec)
    AST_CASE (AST_unary_expression_unary_op)
    AST_CASE (AST_unary_expression_sizeof_exp)
    AST_CASE (AST_unary_expression_sizeof_type)
    AST_CASE (AST_unary_operator_addr)
    AST_CASE (AST_unary_operator_deref)
    AST_CASE (AST_unary_operator_add)
    AST_CASE (AST_unary_operator_sub)
    AST_CASE (AST_unary_operator_comp)
    AST_CASE (AST_unary_operator_neg)
    AST_CASE (AST_postfix_expression_direct)
    AST_CASE (AST_postfix_expression_array)
    AST_CASE (AST_postfix_expression_funcall)
    AST_CASE (AST_postfix_expression_funcall_noarg)
    AST_CASE (AST_postfix_expression_dot)
    AST_CASE (AST_postfix_expression_arrow)
    AST_CASE (AST_postfix_expression_inc)
    AST_CASE (AST_postfix_expression_dec)
    AST_CASE (AST_primary_expression_ID)
    AST_CASE (AST_primary_expression_constant)
    AST_CASE (AST_primary_expression_STRING)
    AST_CASE (AST_primary_expression_paren)
    AST_CASE (AST_argument_expression_list_single)
    AST_CASE (AST_argument_expression_list_pair)
    AST_CASE (AST_constant_INT)
    AST_CASE (AST_constant_CHAR)
    AST_CASE (AST_constant_FLOAT)
    AST_CASE (AST_constant_ENUM)
    AST_CASE (AST_STRING)
    AST_CASE (AST_IDENTIFIER)
    AST_CASE (AST_ENUMERATION_CONSTANT)
    AST_CASE (AST_TYPEDEF_NAME)
    AST_CASE (AST_CHARACTER_CONSTANT)
    AST_CASE (AST_INTEGER_CONSTANT)
    AST_CASE (AST_FLOATING_CONSTANT)
    }
#undef AST_CASE
}
