/*
    XCI: eXperimental C Interpreter
    -- Copyright (c) 2000-2001, gondow@jais.ac.jp
    $Id: AST.h,v 1.1.1.1 2002/06/06 01:41:56 gondow Exp $
 */ 

#ifndef XCI_AST_H
#define XCI_AST_H

#include "attr.h"

/* AST_Type enumerates all kinds of nodes in abstract syntax trees (AST) */

enum AST_Type {
    AST_translation_unit_single,
    AST_translation_unit_pair,

    AST_external_declaration_func,
    AST_external_declaration_dec,

    AST_function_definition_spec,
    AST_function_definition_nospec,

    AST_declaration_list_opt_null,
    AST_declaration_list_opt_direct,

    AST_declaration_direct,

    AST_init_declarator_list_opt_null,
    AST_init_declarator_list_opt_direct,

    AST_declaration_list_single,
    AST_declaration_list_pair,

    AST_declaration_specifers_storage,
    AST_declaration_specifers_storage_pair,
    AST_declaration_specifers_spec,
    AST_declaration_specifers_spec_pair,
    AST_declaration_specifers_qual,
    AST_declaration_specifers_qual_pair,

    AST_storage_class_specifier_auto,
    AST_storage_class_specifier_register,
    AST_storage_class_specifier_static,
    AST_storage_class_specifier_extern,
    AST_storage_class_specifier_typedef,

    AST_type_specifier_void,
    AST_type_specifier_char,
    AST_type_specifier_short,
    AST_type_specifier_int,
    AST_type_specifier_long,
    AST_type_specifier_float,
    AST_type_specifier_double,
    AST_type_specifier_signed,
    AST_type_specifier_unsigned,
    AST_type_specifier_struct_or_union,
    AST_type_specifier_enum,
    AST_type_specifier_TYPEDEF_NAME,

    AST_type_qualifier_const,
    AST_type_qualifier_volatile,

    AST_struct_or_union_specifier_notag,
    AST_struct_or_union_specifier_tag,
    AST_struct_or_union_specifier_incomplete,

    AST_struct_or_union_struct,
    AST_struct_or_union_union,

    AST_struct_declaration_list_single,
    AST_struct_declaration_list_pair,

    AST_init_declarator_single,
    AST_init_declarator_pair,

    AST_init_declarator_dec,
    AST_init_declarator_dec_init,

    AST_struct_declaration_direct,

    AST_specifier_qualifier_list_spec,
    AST_specifier_qualifier_list_spec_pair,
    AST_specifier_qualifier_list_qual,
    AST_specifier_qualifier_list_qual_pair,

    AST_struct_declarator_list_single,
    AST_struct_declarator_list_pair,

    AST_struct_declarator_direct,
    AST_struct_declarator_bitfield_noname,
    AST_struct_declarator_bitfield,

    AST_enum_specifier_untag,
    AST_enum_specifier_tag,
    AST_enum_specifier_dec,

    AST_enumerator_list_single,
    AST_enumerator_list_pair,

    AST_enumerator_id,
    AST_enumerator_id_init,

    AST_declarator_direct,

    AST_pointer_opt_null,
    AST_pointer_opt_direct,

    AST_direct_declarator_id,
    AST_direct_declarator_paren,
    AST_direct_declarator_array,
    AST_direct_declarator_func_new,
    AST_direct_declarator_func_old,

    AST_function_declarator_direct,

    AST_direct_function_declarator_func_new,
    AST_direct_function_declarator_func_old,

    AST_constant_expression_opt_null,
    AST_constant_expression_opt_direct,

    AST_identifier_list_opt_null,
    AST_identifier_list_opt_direct,

    AST_pointer_single,
    AST_pointer_pair,

    AST_type_qualifier_list_opt_nil,
    AST_type_qualifier_list_opt_direct,

    AST_type_qualifier_list_single,
    AST_type_qualifier_list_pair,

    AST_parameter_type_list_direct,
    AST_parameter_type_list_ellipsis,

    AST_parameter_list_single,
    AST_parameter_list_pair,

    AST_parameter_declaration_dec,
    AST_parameter_declaration_abs,

    AST_abstract_declarator_opt_null,
    AST_abstract_declarator_opt_direct,

    AST_identifier_list_single,
    AST_identifier_list_pair,

    AST_initializer_assign,
    AST_initializer_list,
    AST_initializer_list_comma,

    AST_initializer_list_single,
    AST_initializer_list_pair,

    AST_type_name_direct,

    AST_abstract_declarator_pointer,
    AST_abstract_declarator_abs,

    AST_direct_abstract_declarator_paren,
    AST_direct_abstract_declarator_array_nil,
    AST_direct_abstract_declarator_array_pair,
    AST_direct_abstract_declarator_func_nil,
    AST_direct_abstract_declarator_func_pair,

    AST_parameter_type_list_opt_null,
    AST_parameter_type_list_opt_direct,

    AST_statement_labeled,
    AST_statement_expression,
    AST_statement_compound,
    AST_statement_selection,
    AST_statement_iteration,
    AST_statement_jump,

    AST_labeled_statement_label,
    AST_labeled_statement_case,
    AST_labeled_statement_default,

    AST_expression_statement_direct,

    AST_expression_opt_null,
    AST_expression_opt_direct,

    AST_compound_statement_decn,
    AST_compound_statement_nodecn,

    AST_statement_list_opt_null,
    AST_statement_list_opt_direct,

    AST_statement_list_single,
    AST_statement_list_pair,

    AST_selection_statement_if,
    AST_selection_statement_if_else,
    AST_selection_statement_switch,

    AST_iteration_statement_while,
    AST_iteration_statement_do_while,
    AST_iteration_statement_for,

    AST_jump_statement_goto,
    AST_jump_statement_continue,
    AST_jump_statement_break,
    AST_jump_statement_return,

    AST_expression_direct,
    AST_expression_comma,

    AST_assignment_expression_direct,
    AST_assignment_expression_assign,

    AST_assignment_operator_eq,
    AST_assignment_operator_mult,
    AST_assignment_operator_div,
    AST_assignment_operator_mod,
    AST_assignment_operator_add,
    AST_assignment_operator_sub,
    AST_assignment_operator_lshift,
    AST_assignment_operator_rshift,
    AST_assignment_operator_band,
    AST_assignment_operator_beor,
    AST_assignment_operator_bior,

    AST_conditional_expression_direct,
    AST_conditional_expression_cond,

    AST_constant_expression_direct,

    AST_logical_or_expression_direct,
    AST_logical_or_expression_lor,

    AST_logical_and_expression_direct,
    AST_logical_and_expression_land,

    AST_inclusive_or_expression_direct,
    AST_inclusive_or_expression_bior,

    AST_exclusive_or_expression_direct,
    AST_exclusive_or_expression_beor,

    AST_and_expression_direct,
    AST_and_expression_band,

    AST_equality_expression_direct,
    AST_equality_expression_eq,
    AST_equality_expression_neq,

    AST_relational_expression_direct,
    AST_relational_expression_less,
    AST_relational_expression_more,
    AST_relational_expression_lesseq,
    AST_relational_expression_moreeq,

    AST_shift_expression_direct,
    AST_shift_expression_lshift,
    AST_shift_expression_rshift,

    AST_additive_expression_direct,
    AST_additive_expression_add,
    AST_additive_expression_sub,

    AST_multiplicative_expression_direct,
    AST_multiplicative_expression_mult,
    AST_multiplicative_expression_div,
    AST_multiplicative_expression_mod,

    AST_cast_expression_direct,
    AST_cast_expression_cast,

    AST_unary_expression_direct,
    AST_unary_expression_inc,
    AST_unary_expression_dec,
    AST_unary_expression_unary_op,
    AST_unary_expression_sizeof_exp,
    AST_unary_expression_sizeof_type,

    AST_unary_operator_addr,
    AST_unary_operator_deref,
    AST_unary_operator_add,
    AST_unary_operator_sub,
    AST_unary_operator_comp,
    AST_unary_operator_neg,

    AST_postfix_expression_direct,
    AST_postfix_expression_array,
    AST_postfix_expression_funcall,
    AST_postfix_expression_funcall_noarg,
    AST_postfix_expression_dot,
    AST_postfix_expression_arrow,
    AST_postfix_expression_inc,
    AST_postfix_expression_dec,

    AST_primary_expression_ID,
    AST_primary_expression_constant,
    AST_primary_expression_STRING,
    AST_primary_expression_paren,

    AST_argument_expression_list_single,
    AST_argument_expression_list_pair,

    AST_constant_INT,
    AST_constant_CHAR,
    AST_constant_FLOAT,
    AST_constant_ENUM,

    AST_STRING,
    AST_IDENTIFIER,
    AST_ENUMERATION_CONSTANT,
    AST_TYPEDEF_NAME,
    AST_CHARACTER_CONSTANT,
    AST_INTEGER_CONSTANT,
    AST_FLOATING_CONSTANT
};

/* node structure for AST */

struct AST {
    enum AST_Type     ast_type;   /* AST type of this node */
    struct AST	      *parent;    /* back pointer for parent. NULL for root */
    int	       	      nth;	  /* this AST is the nth child of the parent */
    int               num_child;  /* num of children */
    struct Attribute  *attr;      /* attributes to store derived values */
    unsigned int      status;	  /* INTEGER/FLOATING_CONSTANT_* (see below) */

    union {
	struct AST    **child;	  /* array of pointers to child */
	char          *string;    /* identifier, typedef_name, enum constant,
				     string literal */
	int    	      char_i;	  /* character constant */
	unsigned long int_ul;     /* integer constant */
	long double   *float_ld;  /* floating constant */
    } u;

    struct Type       *type;         /* type for expression */
    struct Type       *prom_type;    /* promoted type for expression */
    char              *filename;     /* filename that AST belongs to */
    char              *inc_filename; /* filename that AST belongs to */
    unsigned int      lineno;	     /* last line number that AST belongs to */
};

enum {
    INTEGER_CONSTANT_OCTAL        = 0x01U,   /* 0123 */
    INTEGER_CONSTANT_DECIMAL      = 0x02U,   /* 123  */
    INTEGER_CONSTANT_HEXADECIMAL  = 0x04U,   /* 0xff */
    INTEGER_CONSTANT_UNSIGNED     = 0x08U,   /* 'U' or 'u' suffixed */
    INTEGER_CONSTANT_LONG         = 0x10U    /* 'L' or 'l' suffixed */
};

enum {
    FLOATING_CONSTANT_FLOAT       = 0x01U,   /* 'F' or 'f' suffixed */
    FLOATING_CONSTANT_LONG        = 0x02U    /* 'L' or 'l' suffixed */
};

/*
  we do not use Macro Varargs (see gcc info) here, 
  which is a part of C99 standard,
  because we want the program to be compiled with -ansi option
  */

#define ANULL	     	     	        ((struct AST*) NULL)
#define create_AST1(a1)                 create_AST_ (a1,  ANULL)
#define create_AST2(a1, a2)             create_AST_ (a1, a2, ANULL)
#define create_AST3(a1, a2, a3)         create_AST_ (a1, a2, a3, ANULL)
#define create_AST4(a1, a2, a3, a4)     create_AST_ (a1, a2, a3, a4, ANULL)
#define create_AST5(a1, a2, a3, a4, a5) create_AST_ (a1, a2, a3, a4, a5, ANULL)

struct AST *create_AST_ (enum AST_Type ast_type, ...);

int is_AST_Type_LEAF                (enum AST_Type ast_type);
int is_AST_LEAF                     (struct AST const *ast);
int is_AST_declaration_specifiers   (struct AST const *ast);
int is_AST_specifier_qualifier_list (struct AST const *ast);
int is_AST_expression               (struct AST const *ast);

int is_AST_break_exit               (struct AST const *ast);
int is_AST_continue_exit            (struct AST const *ast);
int is_AST_function_def             (struct AST const *ast);
int is_AST_compound_statement       (struct AST const *ast);

char *asttype2str (enum AST_Type ast_type);

#endif /* XCI_AST_H */
