/*
    XCI: eXperimental C Interpreter for ANSI C
    -- Copyright (c) 2000-2002, gondow@jaist.ac.jp
    $Id: builtin.c,v 1.10 2002/07/09 16:13:35 gondow Exp $
*/ 

#include <stdio.h>
#include <assert.h>
#include <malloc.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include <setjmp.h>
#include <math.h>
#include <time.h>

#include "AST.h"
#include "xci.h"
#include "type.h"
#include "symbol.h"
#include "vm.h"

#define BUILTIN_FILENAME    "**builtin_file**"

/* ---------------------------------------------------------------------- */
/* for <stdio.h> */
static void xci_printf   (void);
static void xci_sprintf  (void);
static void xci_fprintf  (void);
static void xci_vprintf  (void);
static void xci_vfprintf (void);
static void xci_vsprintf (void);
static void xci_fopen    (void);
static void xci_freopen  (void);
static void xci_fflush   (void);
static void xci_fclose   (void);
static void xci_fputs    (void);
static void xci_fgets    (void);
static void xci_fputc    (void);
static void xci_fgetc    (void);
static void xci_ungetc   (void);
static void xci_remove   (void);
static void xci_rename   (void);
static void xci_tmpfile  (void);
static void xci_tmpnam   (void);
static void xci_setvbuf  (void);
static void xci_setbuf   (void);
static void xci_fread    (void);
static void xci_fwrite   (void);
static void xci_ftell    (void);
static void xci_rewind   (void);
static void xci_fseek    (void);
static void xci_fgetpos  (void);
static void xci_fsetpos  (void);
static void xci_clearerr (void);
static void xci_feof     (void);
static void xci_ferror   (void);
static void xci_perror   (void);
static void xci_scanf    (void);
static void xci_fscanf   (void);
static void xci_sscanf   (void);

static void xci_va_start (void);

/* for <ctype.h> */
static void xci_isalnum  (void);
static void xci_isalpha  (void);
static void xci_iscntrl  (void);
static void xci_isdigit  (void);
static void xci_isgraph  (void);
static void xci_islower  (void);
static void xci_isprint  (void);
static void xci_ispunct  (void);
static void xci_isspace  (void);
static void xci_isupper  (void);
static void xci_isxdigit (void);
static void xci_tolower  (void);
static void xci_toupper  (void);

/* for <assert.h> */
static void xci_assert   (void);

/* for <string.h> */
static void xci_memcpy   (void);
static void xci_memmove  (void);
static void xci_strcpy   (void);
static void xci_strncpy  (void);
static void xci_strcat   (void);
static void xci_strncat  (void);
static void xci_memcmp   (void);
static void xci_strcmp   (void);
static void xci_strcoll  (void);
static void xci_strncmp  (void);
static void xci_strxfrm  (void);
static void xci_memchr   (void);
static void xci_strchr   (void);
static void xci_strcspn  (void);
static void xci_strpbrk  (void);
static void xci_strrchr  (void);
static void xci_strspn   (void);
static void xci_strstr   (void);
static void xci_strtok   (void);
static void xci_memset   (void);
static void xci_strerror (void);
static void xci_strlen   (void);

/* for <stdlib.h> */
static void xci_atof     (void);
static void xci_atoi     (void);
static void xci_atol     (void);
static void xci_strtod   (void);
static void xci_strtol   (void);
static void xci_strtoul  (void);
static void xci_rand     (void);
static void xci_srand    (void);
static void xci_calloc   (void);
static void xci_free     (void);
static void xci_malloc   (void);
static void xci_realloc  (void);
static void xci_abort    (void);
static void xci_atexit   (void);
static void xci_exit     (void);
static void xci_getenv   (void);
static void xci_system   (void);
static void xci_bsearch  (void);
static void xci_qsort    (void);
static void xci_abs      (void);
static void xci_labs     (void);
static void xci_div      (void);
static void xci_ldiv     (void);

/* for <math.h> */
static void xci_sin      (void);
static void xci_cos      (void);
static void xci_tan      (void);
static void xci_asin     (void);
static void xci_acos     (void);
static void xci_atan     (void);
static void xci_atan2    (void);
static void xci_sinh     (void);
static void xci_cosh     (void);
static void xci_tanh     (void);
static void xci_exp      (void);
static void xci_log      (void);
static void xci_log10    (void);
static void xci_pow      (void);
static void xci_sqrt     (void);
static void xci_ceil     (void);
static void xci_floor    (void);
static void xci_fabs     (void);
static void xci_frexp    (void);
static void xci_ldexp    (void);
static void xci_modf     (void);
static void xci_fmod     (void);

/* for <setjmp.h> */
static void xci_setjmp   (void);
static void xci_longjmp  (void);

/* for <time.h> */
static void xci_clock    (void);
static void xci_time     (void);
static void xci_difftime (void);
static void xci_mktime   (void);
static void xci_asctime  (void);
static void xci_ctime    (void);
static void xci_gmtime   (void);
static void xci_localtime(void);
static void xci_strftime (void);

/* for debugging */
static void xci_vm_dump  (void);
static void xci_sym_dump (void);
static void xci_trap     (void);

#if 1
static void xci_add (void);
#endif

static FILE *xci_stdin, *xci_stdout, *xci_stderr;

struct BuiltinTable {
    char *name;	   /* identifiers for builtin functions or variables */
    void *addr;	   /* address for the entity */
} builtin_table [] = {
    /* functions in standard libraries */
    /* for <stdio.h> */
    {"printf",   xci_printf},
    {"sprintf",  xci_sprintf},
    {"fprintf",  xci_fprintf},
    {"vprintf",  xci_vprintf},
    {"vfprintf", xci_vfprintf},
    {"vsprintf", xci_vsprintf},
    {"fopen",    xci_fopen},
    {"freopen",  xci_freopen},
    {"fflush",   xci_fflush},
    {"fclose",   xci_fclose},
    {"fgets",    xci_fgets},
    {"fputs",    xci_fputs},
    {"fgetc",    xci_fgetc},
    {"fputc",    xci_fputc},
    {"ungetc",   xci_ungetc},
    {"rename",   xci_rename},
    {"remove",   xci_remove},
    {"tmpfile",  xci_tmpfile},
    {"tmpnam",   xci_tmpnam},
    {"setvbuf",  xci_setvbuf},
    {"setbuf",   xci_setbuf},
    {"fread",    xci_fread},
    {"fwrite",   xci_fwrite},
    {"ftell",    xci_ftell},
    {"rewind",   xci_rewind},
    {"fseek",    xci_fseek},
    {"fgetpos",  xci_fgetpos},
    {"fsetpos",  xci_fsetpos},
    {"clearerr", xci_clearerr},
    {"feof",     xci_feof},
    {"ferror",   xci_ferror},
    {"perror",   xci_perror},
    {"scanf",    xci_scanf},
    {"fscanf",   xci_fscanf},
    {"sscanf",   xci_sscanf},

    /* for <ctype.h> */
    {"isalnum",  xci_isalnum},
    {"isalpha",  xci_isalpha},
    {"iscntrl",  xci_iscntrl},
    {"isdigit",  xci_isdigit},
    {"isgraph",  xci_isgraph},
    {"islower",  xci_islower},
    {"isprint",  xci_isprint},
    {"ispunct",  xci_ispunct},
    {"isspace",  xci_isspace},
    {"isupper",  xci_isupper},
    {"isxdigit", xci_isxdigit},
    {"tolower",  xci_tolower},
    {"toupper",  xci_toupper},

    /* for <assert.h> */
    {"xci_assert",  xci_assert},

    /* for <string.h> */
    {"memcpy",   xci_memcpy},
    {"memmove",  xci_memmove},
    {"strcpy",   xci_strcpy},
    {"strncpy",  xci_strncpy},
    {"strcat",   xci_strcat},
    {"strncat",  xci_strncat},
    {"memcmp",   xci_memcmp},
    {"strcmp",   xci_strcmp},
    {"strcoll",  xci_strcoll},
    {"strncmp",  xci_strncmp},
    {"strxfrm",  xci_strxfrm},
    {"memchr",   xci_memchr},
    {"strchr",   xci_strchr},
    {"strcspn",  xci_strcspn},
    {"strpbrk",  xci_strpbrk},
    {"strrchr",  xci_strrchr},
    {"strspn",   xci_strspn},
    {"strstr",   xci_strstr},
    {"strtok",   xci_strtok},
    {"memset",   xci_memset},
    {"strerror", xci_strerror},
    {"strlen",   xci_strlen},

    /* for <stdlib.h> */
    {"atof",     xci_atof},
    {"atoi",     xci_atoi},
    {"atol",     xci_atol},
    {"strtod",   xci_strtod},
    {"strtol",   xci_strtol},
    {"strtoul",  xci_strtoul},
    {"rand",     xci_rand},
    {"srand",    xci_srand},
    {"calloc",   xci_calloc},
    {"free",     xci_free},
    {"malloc",   xci_malloc},
    {"realloc",  xci_realloc},
    {"abort",    xci_abort},
    {"atexit",   xci_atexit},
    {"exit",     xci_exit},
    {"getenv",   xci_getenv},
    {"system",   xci_system},
    {"bsearch",  xci_bsearch},
    {"qsort",    xci_qsort},
    {"abs",      xci_abs},
    {"labs",     xci_labs},
    {"div",      xci_div},
    {"ldiv",     xci_ldiv},

    /* for <setjmp.h> */
    {"setjmp",   xci_setjmp},
    {"longjmp",  xci_longjmp},

    /* for <math.h> */
    {"sin",      xci_sin},
    {"cos",      xci_cos},
    {"tan",      xci_tan},
    {"asin",     xci_asin},
    {"acos",     xci_acos},
    {"atan",     xci_atan},
    {"atan2",    xci_atan2},
    {"sinh",     xci_sinh},
    {"cosh",     xci_cosh},
    {"tanh",     xci_tanh},
    {"exp",      xci_exp},
    {"log",      xci_log},
    {"log10",    xci_log10},
    {"pow",      xci_pow},
    {"sqrt",     xci_sqrt},
    {"ceil",     xci_ceil},
    {"floor",    xci_floor},
    {"fabs",     xci_fabs},
    {"frexp",    xci_frexp},
    {"ldexp",    xci_ldexp},
    {"modf",     xci_modf},
    {"fmod",     xci_fmod},

    {"clock",    xci_clock},
    {"time",     xci_time},
    {"difftime", xci_difftime},
    {"mktime",   xci_mktime},
    {"asctime",  xci_asctime},
    {"ctime",    xci_ctime},
    {"gmtime",   xci_gmtime},
    {"localtime", xci_localtime},
    {"strftime", xci_strftime},

#if 1 /* for testing and reference */
    {"add",      xci_add},
#endif

    /* for <stdarg.h> */
    {"xci_va_start", xci_va_start},

    /* variables in standard libraries */
    {"stdin",   &xci_stdin},
    {"stdout",  &xci_stdout},
    {"stderr",  &xci_stderr},

    /* for debugging */
    {"vm_dump", xci_vm_dump},
    {"sym_dump", xci_sym_dump},
    {"trap", xci_trap},
    {NULL, NULL}   /* sentinel: this entry must be here in the last */
};

static void sym_entry_builtin (struct BuiltinTable *builtin);
/* ---------------------------------------------------------------------- */
void
register_builtin (void)
{
    int    i;

    sym_begin_file (BUILTIN_FILENAME);
    for (i = 0; builtin_table [i].name != NULL; i++) {
	sym_entry_builtin (&builtin_table [i]);
    }
#if 0 /* debugging */
    {
	void *sp;
	sp = vm.get_sp ();
	vm.push_p ("hogehoge");
	vm.push_d (999.0);
	vm.push_p ("hello, world: %s, %f\n");
	vm.push_p (sp);
	xci_printf ();
	printf ("return value = %d\n", vm.pop_i ());
    }
#endif
    sym_end_file ();

    xci_stdin  = stdin;
    xci_stdout = stdout;
    xci_stderr = stderr;
}

int
is_builtin (void *addr)
{
    int    i;
    for (i = 0; builtin_table [i].name != NULL; i++) {
	if (addr == builtin_table [i].addr)
	    return 1;
    }
    return 0;
}

/* ---------------------------------------------------------------------- */
static void
sym_entry_builtin (struct BuiltinTable *builtin)
{
    struct Symbol *sym, *sym2;

    sym = emalloc (sizeof (struct Symbol));

    sym->name      = builtin->name;
    sym->type      = NULL;
    sym->ast       = NULL;
    sym->addr      = builtin->addr;
    sym->filename  = BUILTIN_FILENAME;
    sym->namespace = NS_NORMAL;
    sym->namelevel = NL_FILE;
    sym->status    = SYM_STAT_DEFINITION | SYM_STAT_GLOBAL | SYM_STAT_BUILTIN;

    sym2 = sym_entry (sym);
    assert (sym2 == sym);
}

#if 0
static void
init_builtin (void)
{
    struct Symbol *sym;

    /* setup stdin */
    sym = sym_lookup_at_all_files ("stdin", NS_NORMAL);
    assert (sym != NULL);
    assert (sym->addr != NULL);
    vm.push_p (stdin);
    vm.pop_b (sym->addr, sizeof (stdin));

    /* setup stdout */
    sym = sym_lookup_at_all_files ("stdout", NS_NORMAL);
    assert (sym != NULL);
    assert (sym->addr != NULL);
    vm.push_p (stdout);
    vm.pop_b (sym->addr, sizeof (stdout));

    /* setup stderr */
    sym = sym_lookup_at_all_files ("stderr", NS_NORMAL);
    assert (sym != NULL);
    assert (sym->addr != NULL);
    vm.push_p (stderr);
    vm.pop_b (sym->addr, sizeof (stderr));
}
#endif

static void
xci_printf (void)
{
    va_list  argp    = vm.pop_p ();
    char     *format = vm.pop_p ();
    int      ret     = vprintf (format, argp);
    vm.set_sp (argp);
    vm.push_i (ret);
}

static void
xci_fprintf (void)
{
    va_list argp    = vm.pop_p ();
    char    *format = vm.pop_p ();
    FILE    *fp     = vm.pop_p ();
    int     ret     = vfprintf (fp, format, argp);
    vm.set_sp (argp);
    vm.push_i (ret);
}

static void
xci_sprintf (void)
{
    va_list  argp    = vm.pop_p ();
    char     *format = vm.pop_p ();
    char     *s      = vm.pop_p ();
    int      ret     = vsprintf (s, format, argp);
    vm.set_sp (argp);
    vm.push_i (ret);
}

static void
xci_vprintf (void)
{
    xci_error_noast ("vprintf not implemented yet");
#if 0
    va_list  argp    = vm.pop_p ();
    char     *format = vm.pop_p ();
    int      ret     = vprintf (format, argp);
    vm.push_i (ret);
#endif
}

static void
xci_vfprintf (void)
{
    xci_error_noast ("vfprintf not implemented yet");
#if 0
    va_list  argp    = vm.pop_p ();
    char     *format = vm.pop_p ();
    FILE     *stream = vm.pop_p ();
    int      ret     = vfprintf (stream, format, argp);
    vm.push_i (ret);
#endif
}

static void
xci_vsprintf (void)
{
    xci_error_noast ("vsprintf not implemented yet");
#if 0
    va_list  argp    = vm.pop_p ();
    char     *format = vm.pop_p ();
    char     *s      = vm.pop_p ();
    int      ret     = vsprintf (s, format, argp);
    vm.push_i (ret);
#endif
}

static void
xci_fopen (void)
{
    char *mode = vm.pop_p ();
    char *filename = vm.pop_p ();
    FILE *ret = fopen (filename, mode);
    vm.push_p (ret);
}

static void
xci_freopen (void)
{
    FILE *stream = vm.pop_p ();
    char *mode = vm.pop_p ();
    char *filename = vm.pop_p ();
    FILE *ret = freopen (filename, mode, stream);
    vm.push_p (ret);
}

static void
xci_fflush (void)
{
    FILE *stream = vm.pop_p ();
    int  ret = fflush (stream);
    vm.push_i (ret);
}

static void
xci_fclose (void)
{
    FILE *fp = vm.pop_p ();
    int ret = fclose (fp);
    vm.push_i (ret);
}

static void
xci_fputs (void)
{
    FILE    *fp     = vm.pop_p ();
    char    *s      = vm.pop_p ();
    int     ret     = fputs (s, fp);
    vm.push_i (ret);
}

static void
xci_fgets (void)
{
    FILE    *fp     = vm.pop_p ();
    int     n       = vm.pop_i ();
    char    *buf    = vm.pop_p ();
    char    *ret    = fgets (buf, n, fp);
    vm.push_p (ret);
}

static void
xci_fputc (void)
{
    FILE    *fp     = vm.pop_p ();
    int     c       = vm.pop_i ();
    int     ret     = fputc (c, fp);
    vm.push_i (ret);
}

static void
xci_fgetc (void)
{
    FILE    *fp     = vm.pop_p ();
    int     ret     = fgetc (fp);
    vm.push_i (ret);
}

static void
xci_ungetc   (void)
{
    FILE *stream = vm.pop_p ();
    int  c       = vm.pop_i ();
    int  ret     = ungetc (c, stream);
    vm.push_i (ret);
}

static void
xci_rename (void)
{
    const char *newname = vm.pop_p ();
    const char *oldname = vm.pop_p ();
    int   ret = rename (oldname, newname);
    vm.push_i (ret);
}

static void
xci_remove (void)
{
    const char *filename = vm.pop_p ();
    int   ret = remove (filename);
    vm.push_i (ret);
}

static void
xci_tmpfile  (void)
{
    FILE *fp = tmpfile ();
    vm.push_p (fp);
}

static void
xci_tmpnam   (void)
{
    char *s = vm.pop_p ();
    char *ret = tmpnam (s);
    vm.push_p (ret);
}

static void
xci_setvbuf  (void)
{
    size_t size  = vm.pop_i ();
    size_t mode  = vm.pop_i ();
    char   *buf  = vm.pop_p ();
    FILE *stream = vm.pop_p ();
    int ret = setvbuf (stream, buf, mode, size);
    vm.push_i (ret);
}

static void
xci_setbuf   (void)
{
    char *buf    = vm.pop_p ();
    FILE *stream = vm.pop_p ();
    setbuf (stream, buf);
}

static void
xci_fread    (void)
{
    FILE *stream = vm.pop_p ();
    size_t nobj  = vm.pop_i ();
    size_t size  = vm.pop_i ();
    void  *ptr   = vm.pop_p ();
    size_t ret   = fread (ptr, size, nobj, stream);
    vm.push_i (ret);
}

static void
xci_fwrite   (void)
{
    FILE *stream    = vm.pop_p ();
    size_t nobj     = vm.pop_i ();
    size_t size     = vm.pop_i ();
    const void *ptr = vm.pop_p ();
    size_t ret      = fwrite (ptr, size, nobj, stream);
    vm.push_i (ret);
}

static void
xci_ftell    (void)
{
    FILE *stream = vm.pop_p ();
    long ret     = ftell (stream);
    vm.push_i (ret);
}

static void
xci_rewind   (void)
{
    FILE *stream = vm.pop_p ();
    rewind (stream);
}

static void
xci_fseek    (void)
{
    int  origin  = vm.pop_i ();
    long offset  = vm.pop_i ();
    FILE *stream = vm.pop_p ();
    int  ret     = fseek (stream, offset, origin);
    vm.push_i (ret);
}

static void
xci_fgetpos  (void)
{
    fpos_t *ptr    = vm.pop_p ();
    FILE   *stream = vm.pop_p ();
    int    ret     = fgetpos (stream, ptr);
    vm.push_i (ret);
}

static void
xci_fsetpos  (void)
{
    fpos_t *ptr    = vm.pop_p ();
    FILE   *stream = vm.pop_p ();
    int    ret     = fsetpos (stream, ptr);
    vm.push_i (ret);
}

static void
xci_clearerr (void)
{
    FILE *fp = vm.pop_p ();
    clearerr (fp);
}

static void
xci_feof     (void)
{
    FILE *fp = vm.pop_p ();
    int  ret = feof (fp);
    vm.push_i (ret);
}

static void
xci_ferror   (void)
{
    FILE *fp = vm.pop_p ();
    int  ret = ferror (fp);
    vm.push_i (ret);
}

static void
xci_perror   (void)
{
    const char *s = vm.pop_p ();
    perror (s);
}

static void
xci_scanf    (void)
{
    xci_error_noast ("vprintf not implemented yet");
}

static void
xci_fscanf   (void)
{
    xci_error_noast ("vprintf not implemented yet");
}

static void
xci_sscanf   (void)
{
    xci_error_noast ("vprintf not implemented yet");
}

static void xci_isalnum  (void) { vm.push_i (isalnum  (vm.pop_i ())); }
static void xci_isalpha  (void) { vm.push_i (isalpha  (vm.pop_i ())); }
static void xci_iscntrl  (void) { vm.push_i (iscntrl  (vm.pop_i ())); }
static void xci_isdigit  (void) { vm.push_i (isdigit  (vm.pop_i ())); }
static void xci_isgraph  (void) { vm.push_i (isgraph  (vm.pop_i ())); }
static void xci_islower  (void) { vm.push_i (islower  (vm.pop_i ())); }
static void xci_isprint  (void) { vm.push_i (isprint  (vm.pop_i ())); }
static void xci_ispunct  (void) { vm.push_i (ispunct  (vm.pop_i ())); }
static void xci_isspace  (void) { vm.push_i (isspace  (vm.pop_i ())); }
static void xci_isupper  (void) { vm.push_i (isupper  (vm.pop_i ())); }
static void xci_isxdigit (void) { vm.push_i (isxdigit (vm.pop_i ())); }
static void xci_tolower  (void) { vm.push_i (tolower  (vm.pop_i ())); }
static void xci_toupper  (void) { vm.push_i (toupper  (vm.pop_i ())); }

static void
xci_assert (void)
{
    const char *expression = vm.pop_p ();
    int   line = vm.pop_i ();
    const char *file = vm.pop_p ();
    fprintf (stderr, "assertion %s failed: file \"%s\", line = %d\n",
	     expression, file, line);
}

static void
xci_memcpy   (void)
{
    size_t n       = vm.pop_i ();
    const void *s2 = vm.pop_p ();
    void *s1       = vm.pop_p ();
    void *ret      = memcpy (s1, s2, n);
    vm.push_p (ret);
}

static void
xci_memmove  (void)
{
    size_t n       = vm.pop_i ();
    const void *s2 = vm.pop_p ();
    void *s1       = vm.pop_p ();
    void *ret      = memcpy (s1, s2, n);
    vm.push_p (ret);
}

static void
xci_strcpy   (void)
{
    const char *s2 = vm.pop_p ();
    char *s1       = vm.pop_p ();
    char *ret      = strcpy (s1, s2);
    vm.push_p (ret);
}

static void
xci_strncpy  (void)
{
    size_t n       = vm.pop_i ();
    const char *s2 = vm.pop_p ();
    char *s1       = vm.pop_p ();
    char *ret      = strncpy (s1, s2, n);
    vm.push_p (ret);
}

static void
xci_strcat   (void)
{
    const char *s2 = vm.pop_p ();
    char *s1       = vm.pop_p ();
    char *ret      = strcat (s1, s2);
    vm.push_p (ret);
}

static void
xci_strncat  (void)
{
    size_t n       = vm.pop_i ();
    const char *s2 = vm.pop_p ();
    char *s1       = vm.pop_p ();
    char *ret      = strncat (s1, s2, n);
    vm.push_p (ret);
}

static void
xci_memcmp   (void)
{
    size_t n       = vm.pop_i ();
    const void *s2 = vm.pop_p ();
    const void *s1 = vm.pop_p ();
    int   ret      = memcmp (s1, s2, n);
    vm.push_i (ret);
}

static void
xci_strcmp   (void)
{
    const char *s2 = vm.pop_p ();
    const char *s1 = vm.pop_p ();
    int   ret      = strcmp (s1, s2);
    vm.push_i (ret);
}

static void
xci_strcoll  (void)
{
    const char *s2 = vm.pop_p ();
    const char *s1 = vm.pop_p ();
    int   ret      = strcoll (s1, s2);
    vm.push_i (ret);
}

static void
xci_strncmp  (void)
{
    size_t n       = vm.pop_i ();
    const char *s2 = vm.pop_p ();
    const char *s1 = vm.pop_p ();
    int   ret      = strncmp (s1, s2, n);
    vm.push_i (ret);
}

static void
xci_strxfrm  (void)
{
    size_t n       = vm.pop_i ();
    const char *s2 = vm.pop_p ();
    char  *s1      = vm.pop_p ();
    size_t ret     = strxfrm (s1, s2, n);
    vm.push_i (ret);
}

static void
xci_memchr   (void)
{
    size_t n      = vm.pop_i ();
    int    c      = vm.pop_i ();
    const void *s = vm.pop_p ();
    void *ret     = memchr (s, c, n);
    vm.push_p (ret);
}

static void
xci_strchr   (void)
{
    int    c      = vm.pop_i ();
    const void *s = vm.pop_p ();
    char *ret     = strchr (s, c);
    vm.push_p (ret);
}

static void
xci_strcspn  (void)
{
    const void *s2 = vm.pop_p ();
    const void *s1 = vm.pop_p ();
    size_t ret     = strcspn (s1, s2);
    vm.push_i (ret);
}

static void
xci_strpbrk  (void)
{
    const void *s2 = vm.pop_p ();
    const void *s1 = vm.pop_p ();
    char  *ret     = strpbrk (s1, s2);
    vm.push_p (ret);
}

static void
xci_strrchr  (void)
{
    int    c      = vm.pop_i ();
    const void *s = vm.pop_p ();
    char *ret     = strrchr (s, c);
    vm.push_p (ret);
}

static void
xci_strspn   (void)
{
    const void *s2 = vm.pop_p ();
    const void *s1 = vm.pop_p ();
    size_t ret     = strspn (s1, s2);
    vm.push_i (ret);
}

static void
xci_strstr   (void)
{
    const void *s2 = vm.pop_p ();
    const void *s1 = vm.pop_p ();
    char  *ret     = strstr (s1, s2);
    vm.push_p (ret);
}

static void
xci_strtok   (void)
{
    const void *s2 = vm.pop_p ();
    void *s1       = vm.pop_p ();
    char  *ret     = strtok (s1, s2);
    vm.push_p (ret);
}

static void
xci_memset   (void)
{
    size_t n  = vm.pop_i ();
    int    c  = vm.pop_i ();
    void  *s  = vm.pop_p ();
    void *ret = memset (s, c, n);
    vm.push_p (ret);
}

static void
xci_strerror (void)
{
    int ret_arg = vm.pop_i ();
    char *ret   = strerror (ret_arg);
    vm.push_p (ret);
}

static void
xci_strlen   (void)
{
    const char *s = vm.pop_p ();
    size_t ret    = strlen (s);
    vm.push_i (ret);
}

static void
xci_atof     (void)
{
    const char *nptr = vm.pop_p ();
    double ret       = atof (nptr);
    vm.push_d (ret);
}

static void
xci_atoi (void)
{
    const char *s = vm.pop_p ();
    int  ret      = atoi (s);
    vm.push_i (ret);
}

static void
xci_atol     (void)
{
    const char *nptr = vm.pop_p ();
    long  ret        = atol (nptr);
    vm.push_i (ret);
}

static void
xci_strtod   (void)
{
    char **endptr    = vm.pop_p ();
    const char *nptr = vm.pop_p ();
    double  ret      = strtod (nptr, endptr);
    vm.push_d (ret);
}

static void
xci_strtol   (void)
{
    int  base        = vm.pop_i ();
    char **endptr    = vm.pop_p ();
    const char *nptr = vm.pop_p ();
    long  ret        = strtol (nptr, endptr, base);
    vm.push_i (ret);
}

static void
xci_strtoul  (void)
{
    int  base          = vm.pop_i ();
    char **endptr      = vm.pop_p ();
    const char *nptr   = vm.pop_p ();
    unsigned long  ret = strtoul (nptr, endptr, base);
    vm.push_i (ret);
}

static void
xci_rand (void)
{
    int ret = rand ();
    vm.push_i (ret);
}

static void
xci_srand (void)
{
    int  arg = vm.pop_i ();
    srand (arg);
}

static void
xci_calloc   (void)
{
    size_t size  = vm.pop_i ();
    size_t nmemb = vm.pop_i ();
    void *ret    = calloc (nmemb, size);
    vm.push_p (ret);
}

static void
xci_free     (void)
{
    void *ptr = vm.pop_p ();
    free (ptr);
}

static void
xci_malloc   (void)
{
    size_t size = vm.pop_i ();
    void *ret = malloc (size);
    vm.push_p (ret);
}

static void
xci_realloc  (void)
{
    size_t size = vm.pop_i ();
    void *ptr   = vm.pop_p ();
    void *ret   = realloc (ptr, size);
    vm.push_p (ret);
}

static void
xci_abort    (void)
{
    abort ();
}

static void
xci_atexit   (void)
{
    void *func = vm.pop_p ();
    int ret;

    if (atexit_index >= MAX_ATEXIT_FUNC - 1) {
	ret = 1;
    } else {
	atexit_func [atexit_index] = func;
	atexit_index++;
	ret = 0;
    }
    vm.push_i (ret);
}

static void
xci_exit (void)
{
    int status = vm.pop_i ();
    exit (status);
}

static void
xci_getenv   (void)
{
    const char *name = vm.pop_p ();
    char *ret = getenv (name);
    vm.push_p (ret);
}

static void
xci_system   (void)
{
    const char *string = vm.pop_p ();
    int ret = system (string);
    vm.push_i (ret);
}

static void
xci_bsearch  (void)
{
    xci_error_noast ("bsearch not implemented yet");
#if 0
    int (*compar)(const void *, const void *) = vm.pop_p ();
    size_t size = vm.pop_i ();
    size_t nmemb = vm.pop_i ();
    const void *base = vm.pop_p ();
    const void *key = vm.pop_p ();
    void *ret = bsearch (key, base, nmemb, size, compar);
    vm.push_p (ret);
#endif
}

static void
xci_qsort    (void)
{
    xci_error_noast ("qsort not implemented yet");
#if 0
    int (*compar)(const void *, const void *) = vm.pop_p ();
    size_t size  = vm.pop_i ();
    size_t nmemb = vm.pop_i ();
    void *base   = vm.pop_p ();
    qsort (base, nmemb, size, compar);
#endif
}

static void
xci_abs      (void)
{
    int j = vm.pop_i ();
    int ret = abs (j);
    vm.push_i (ret);
}

static void
xci_labs     (void)
{
    long j = vm.pop_i ();
    long ret = labs (j);
    vm.push_i (ret);
}

static void
xci_div      (void)
{
    int denom = vm.pop_i ();
    int number = vm.pop_i ();
    div_t ret  = div (number, denom);
    vm.push_b (&ret, sizeof (div_t));
}

static void
xci_ldiv     (void)
{
    long denom = vm.pop_i ();
    long number = vm.pop_i ();
    ldiv_t ret  = ldiv (number, denom);
    vm.push_b (&ret, sizeof (ldiv_t));
}

#if 0
struct xci_jmp_buf {
    void *sp, *bp, *ast;
    int nth;
};

jmp_buf native_jb;
#endif

static void
xci_setjmp   (void)
{
    xci_error_noast ("setjmp not implemented yet");
#if 0
    struct xci_jmp_buf *xci_jb = vm.pop_p ();
    xci_jb->sp  =  vm.get_sp (); /* stack pointer */
    xci_jb->bp  =  vm.get_bp (); /* frame pointer */
    xci_jb->ast =  (void *) vm.pc.ast;    /* program counter (ast) */
    xci_jb->nth =  vm.pc.nth;    /* program counter (nth) */
    printf ("setjmp: sp = %p, bp = %p, ast = %p, nth = %d\n",
	    xci_jb->sp, xci_jb->bp, xci_jb->ast, xci_jb->nth);
    if (setjmp (native_jb) == 0)
	vm.push_i (0);
#endif
}

static void
xci_longjmp  (void)
{
    xci_error_noast ("longjmp not implemented yet");
#if 0
    int val = vm.pop_i ();
    struct xci_jmp_buf *jb = vm.pop_p ();
    printf ("longjmp: sp = %p, bp = %p, ast = %p, nth = %d, val = %d\n",
	    jb->sp, jb->bp, jb->ast, jb->nth, val);
    vm.set_sp (jb->sp);
    vm.set_bp (jb->bp);
    vm.set_pc (jb->ast, jb->nth);
    vm.push_i (val);
    longjmp (native_jb, 1);
#endif
}

static void xci_sin   (void) { vm.push_d (sin  (vm.pop_d ())); }
static void xci_cos   (void) { vm.push_d (cos  (vm.pop_d ())); }
static void xci_tan   (void) { vm.push_d (tan  (vm.pop_d ())); }
static void xci_asin  (void) { vm.push_d (asin (vm.pop_d ())); }
static void xci_acos  (void) { vm.push_d (acos (vm.pop_d ())); }
static void xci_atan  (void) { vm.push_d (atan (vm.pop_d ())); }
static void xci_sinh  (void) { vm.push_d (sinh (vm.pop_d ())); }
static void xci_cosh  (void) { vm.push_d (cosh (vm.pop_d ())); }
static void xci_tanh  (void) { vm.push_d (tanh (vm.pop_d ())); }

static void
xci_atan2    (void)
{
    double y   = vm.pop_d ();
    double x   = vm.pop_d ();
    double ret = atan2 (x, y);
    vm.push_d (ret);
}

static void xci_exp   (void) { vm.push_d (exp   (vm.pop_d ())); }
static void xci_log   (void) { vm.push_d (log   (vm.pop_d ())); }
static void xci_log10 (void) { vm.push_d (log10 (vm.pop_d ())); }
static void xci_sqrt  (void) { vm.push_d (sqrt  (vm.pop_d ())); }

static void
xci_pow      (void)
{
    double y   = vm.pop_d ();
    double x   = vm.pop_d ();
    double ret = pow (x, y);
    vm.push_d (ret);
}

static void xci_ceil  (void) { vm.push_d (ceil  (vm.pop_d ())); }
static void xci_floor (void) { vm.push_d (floor (vm.pop_d ())); }
static void xci_fabs  (void) { vm.push_d (fabs  (vm.pop_d ())); }

static void
xci_frexp (void)
{
    int *exp   = vm.pop_p ();
    double x   = vm.pop_d ();
    double ret = frexp (x, exp);
    vm.push_d (ret);
}

static void
xci_ldexp    (void)
{
    int exp  = vm.pop_i ();
    double x = vm.pop_d ();
    double ret = ldexp (x, exp);
    vm.push_d (ret);
}

static void
xci_modf     (void)
{
    double *iptr = vm.pop_p ();
    double value = vm.pop_d ();
    double ret   = modf (value, iptr);
    vm.push_d (ret);
}

static void
xci_fmod     (void)
{
    double y   = vm.pop_d ();
    double x   = vm.pop_d ();
    double ret = fmod (x, y);
    vm.push_d (ret);
}

static void
xci_clock    (void)
{
    clock_t ret = clock ();
    vm.push_i (ret);
}

static void
xci_time     (void)
{
    time_t *timer = vm.pop_p ();
    time_t ret    = time (timer);
    vm.push_i (ret);
}

static void
xci_difftime (void)
{
    time_t time0 = vm.pop_i ();
    time_t time1 = vm.pop_i ();
    double ret   = difftime (time1, time0);
    vm.push_d (ret);
}

static void
xci_mktime   (void)
{
    struct tm *timeptr = vm.pop_p ();
    time_t ret = mktime (timeptr);
    vm.push_d (ret);
}

static void
xci_asctime  (void)
{
    const struct tm *timeptr = vm.pop_p ();
    char *ret = asctime (timeptr);
    vm.push_p (ret);
}

static void
xci_ctime    (void)
{
    const time_t *timer = vm.pop_p ();
    char *ret = ctime (timer);
    vm.push_p (ret);
}

static void
xci_gmtime   (void)
{
    const time_t *timer = vm.pop_p ();
    struct tm *ret = gmtime (timer);
    vm.push_p (ret);
}

static void
xci_localtime(void)
{
    const time_t *timer = vm.pop_p ();
    struct tm *ret = localtime (timer);
    vm.push_p (ret);
}

static void
xci_strftime (void)
{
    const struct tm *timeptr = vm.pop_p ();
    const char *format = vm.pop_p ();
    size_t maxsize = vm.pop_i ();
    char *s = vm.pop_p ();
    size_t ret = strftime (s, maxsize, format, timeptr);
    vm.push_i (ret);
}


static void
xci_va_start (void)
{
    char   *lastarg    = vm.pop_p ();
    struct Symbol *sym = sym_lookup (lastarg, NS_NORMAL);
    assert (sym != NULL);
    assert (sym->addr != NULL);
    vm.push_p (sym->addr);
}
/* ---------------------------------------------------------------------- */
static void
xci_vm_dump (void)
{
    vm.dump ();
}

static void
xci_sym_dump (void)
{
    symtable_dump (0);
}

static void
xci_trap (void)
{
#if 0
    asm ("int3");
#endif
}

#if 1
static void
xci_add (void)
{
    int ret;
    int b = vm.pop_i ();
    int a = vm.pop_i ();
    ret = a + b;
    printf ("a = %d\n", a);
    printf ("b = %d\n", b);
    vm.push_i (ret);
}
#endif
/* ---------------------------------------------------------------------- */
