/*
    XCI: eXperimental C Interpreter for ANSI C
    -- Copyright (c) 2000, gondow@jaist.ac.jp
    $Id: type.h,v 1.1.1.1 2002/06/06 01:41:56 gondow Exp $
*/ 

#ifndef XCI_TYPE_H
#define XCI_TYPE_H

enum DeclContext {
    DECL_CONTEXT_GLOBAL,
    DECL_CONTEXT_LOCAL
#if 0	 /* not used */
    DECL_CONTEXT_UNKNOWN,
    DECL_CONTEXT_ARGUMENT,
    DECL_CONTEXT_FUNCTION
#endif
};

enum PrimType {
    PRIM_TYPE_VOID,
    PRIM_TYPE_CHAR,
    PRIM_TYPE_INT,
    PRIM_TYPE_FLOAT,
    PRIM_TYPE_DOUBLE
};

enum TypeModifier {
    TYPE_MOD_NONE        = 0x00U,
    TYPE_MOD_LONG        = 0x01U,    /* modifier 'long' */
    TYPE_MOD_SHORT       = 0x02U,    /* modifier 'short' */
    TYPE_MOD_SIGNED      = 0x04U,    /* modifier 'signed' */
    TYPE_MOD_UNSIGNED    = 0x08U     /* modifier 'unsigned' */
#ifdef XCI_LONGLONG
    /* Note: 'long long' is illegal in ANSI C, but legal in C99. */
    ,TYPE_MOD_LONGLONG   = 0x10U     /* modifier 'long long' */
    /* Note: C99 accepts extra commas in enumerator-list. */
#endif /* XCI_LONGLONG */
};

enum TypeQualifier {
    TYPE_QUAL_NONE       = 0x00U,
    TYPE_QUAL_CONST      = 0x01U,    /* qualifier 'const' */
    TYPE_QUAL_VOLATILE	 = 0x02U     /* qualifier 'volatile' */
};

enum StorageClass {
    STORAGE_NONE,
    STORAGE_REGISTER,
    STORAGE_AUTO,
    STORAGE_STATIC,
    STORAGE_EXTERN,
    STORAGE_TYPEDEF
};

enum TypeKind {
    TYPE_KIND_PRIM,         /* primitive type */
    TYPE_KIND_ENUM,         /* enumeration type */
    TYPE_KIND_STRUCT,       /* structure (Cartesian product) */
    TYPE_KIND_UNION,        /* union (direct sum) */
    TYPE_KIND_FUNCTION,     /* function type */
    TYPE_KIND_POINTER,      /* pointer type */
    TYPE_KIND_ARRAY,        /* array type */
    TYPE_KIND_QUAL          /* type qualifier: const, volatile */
};

struct StructUnionField {
    char        *name;
    struct Type *type;
    int	    	offset;
    int    	is_bit_field;
    int         bit_offset;	  /* for bit-fields */
    int    	bit_size;	  /* for bit-fields */
};

struct Type {
    enum TypeKind   kind;
    int             size;	  /* total size of this type */
    
    union {
	/* this must be first for initialization */
	struct {
	    int            modifier; /* combination of enum TypeModifier */
	    enum PrimType  ptype;
    	} t_prim;

	struct {
	    char         *tag;
	    int          num;
	    char         **elem; /* array of names */
	    int          *value; /* array of values */
    	} t_enum;

    	struct {
	    char   *tag;
	    int    num;
	    struct StructUnionField **field;
    	} t_struct_union;

    	struct {
	    struct Type *ret_type;
	    int         is_ellipsis;	/* ", ..."; see stdarg.h */
	    int         is_old_type;    /* old-type arguments; without types */
	    int      	arg_num;    	/* not including ", ..." */
	    char        **arg_name;     /* array of argument names */
	    struct Type **arg_type;     /* array of argument type pointers */
    	} t_function;

    	struct {
	    struct Type *type;
    	} t_pointer;

    	struct {
	    struct Type *elem_type;
	    int         num;
    	} t_array;

	struct {
    	    int qualifier;        /* combination of enum TypeQualifier */
	    struct Type *type;    /* qualifying type */
	} t_qual;
    } u;
};

#define ANONYMOUS_FIELD	   "**anonymous_field**"

/* ---------------------------------------------------------------------- */

extern enum DeclContext   decl_context;

void   type_analyze        (struct AST *ast);
void   type_analyze_exp    (struct AST *ast);
void   check_main_type     (void);
int    is_type_compatible  (struct Type *type1, struct Type *type2);
int    is_sym_stat_compatible (int stat1, int stat2);
int    create_sym_stat     (struct AST *ast, struct Type *type,
			    char const * const id, enum StorageClass storage);

int    get_type_size       (struct Type *type);
struct Type *get_type      (struct AST *ast);

int    is_void_type        (struct Type *type);
int    is_integral_type    (struct Type *type);
int    is_floating_type    (struct Type *type);
int    is_arithmetic_type  (struct Type *type);
int    is_void_ast         (struct AST  *ast);
int    is_integral_ast     (struct AST  *ast);
int    is_floating_ast     (struct AST  *ast);
int    is_arithmetic_ast   (struct AST  *ast);

int    is_type_equivalent (struct Type *type1, struct Type *type2);

void type_dump (struct Type *type, int depth);

#endif /* XCI_TYPE_H */
