/*
    XCI: eXperimental C Interpreter for ANSI C
    -- Copyright (c) 2000-2001, gondow@jais.ac.jp
    $Id: vm.h,v 1.2 2002/07/09 11:43:46 gondow Exp $
 */ 

#ifndef XCI_VM_H
#define XCI_VM_H
#include "AST.h"
#include "type.h"

enum StackType {
    ST_CHAR,
    ST_INT,
    ST_DOUBLE,
    ST_POINTER,
    ST_STRING,
    ST_OTHER
};


/* virtual machine interface */
struct VM {
    enum VMStat {
	VM_STAT_STOPPED, VM_STAT_RUNNING
    } status;
    void   *sp;                      /* stack pointer */
    void   *bp;                      /* base  pointer */
    struct PC {
	struct AST const *ast;
	int        nth;
    }      pc;                       /* program counter */

    void   *stack;                   /* stack */
    char   *stack_type;              /* stack type */
    int    stack_size;

    void   (*init)   (int stack_size);     /* initialize virtual machine */

    /* stack manipulating */
    void   (*push_c) (char c);	     /* push char    */
    char   (*pop_c)  (void);         /* pop  char    */
    char   (*top_c)  (void);         /* top  char    */
    void   (*push_i) (int i);        /* push int     */
    int    (*pop_i)  (void);         /* pop  int     */
    int    (*top_i)  (void);         /* top  int     */
    void   (*push_d) (double d);     /* push double  */
    double (*pop_d)  (void);         /* pop  double  */
    double (*top_d)  (void);         /* top  double  */
    void   (*push_p) (void *p);      /* push pointer */
    void*  (*pop_p)  (void);         /* pop  pointer */
    void*  (*top_p)  (void);         /* top  pointer */

    void   (*push_b) (void *from_addr, int size);
    void   (*pop_b)  (void *to_addr,   int size);
    void   (*top_b)  (void *to_addr,   int size);

/*
    void   (*move)   (char *src, char *dest, int size);
*/

    void   (*add_sp) (int size);
    void   (*sub_sp) (int size);
    void*  (*get_sp) (void);
    void   (*set_sp) (void *sp);

    void*  (*get_bp) (void);
    void   (*set_bp) (void *bp);

    void   (*set_pc) (void *ast, int nth);
    
    /* allocation */
/*
    void   (*static_alloc) (int size);
    void   (*heap_alloc)   (int size);
    void   (*stack_alloc)  (int size);
*/

    void   (*dump)    (void);        /* for debug */
    int    (*size)    (struct Type *type);

    /* arithmetic operators: */
    void   (*add_i)   (void);
    void   (*sub_i)   (void);
    void   (*mult_i)  (void);
    void   (*div_i)   (void);
    void   (*mod_i)   (void);

    void   (*add_d)   (void);
    void   (*sub_d)   (void);
    void   (*mult_d)  (void);
    void   (*div_d)   (void);

    /* logical operators */
    void   (*l_not)   (void);
    /* l_and and l_or are not a part of vm. */

    /* relational operators */
    void   (*less_i)  (void);
    void   (*leq_i)   (void);
    void   (*more_i)  (void);
    void   (*meq_i)   (void);
    void   (*eq_i)    (void);
    void   (*neq_i)   (void);

    void   (*less_d)  (void);
    void   (*leq_d)   (void);
    void   (*more_d)  (void);
    void   (*meq_d)   (void);
    void   (*eq_d)    (void);
    void   (*neq_d)   (void);

    /* bit operators */

    void   (*l_sh)    (void);
    void   (*r_sh)    (void);
    void   (*r_ush)   (void);
    void   (*b_and)   (void);
    void   (*b_or)    (void);
    void   (*b_xor)   (void);
    void   (*b_comp)  (void);

    /* type casting */
    void   (*c_to_i)  (void);    /* char    -> int     */
    void   (*i_to_c)  (void);    /* int     -> char    */
    void   (*i_to_d)  (void);    /* int     -> double  */
    void   (*d_to_i)  (void);    /* double  -> int     */
    void   (*p_to_i)  (void);    /* pointer -> int     */
    void   (*i_to_p)  (void);    /* int     -> pointer */

#if 0
    /* jump */
    void   (*jump)    (void);    /* ast, nth -> (none) */
    void   (*jump_if) (void);    /* ast, nth, cond -> (none) */

    /* subroutine call/return */
    void   (*call)    (void);
    void   (*ret)     (void);
#endif

    /* constants */
    long     VM_CHAR_MAX;
    long     VM_CHAR_MIN;
    long     VM_INT_MAX;
    long     VM_INT_MIN;
    long     VM_LONG_MAX;
    long     VM_LONG_MIN;
    long     VM_SCHAR_MAX;
    long     VM_SCHAR_MIN;
    long     VM_SHRT_MAX;
    long     VM_SHRT_MIN;
    unsigned long VM_UCHAR_MAX;
    unsigned long VM_UINT_MAX;
    unsigned long VM_ULONG_MAX;
    unsigned long VM_USHRT_MAX;

    int      VM_SIZEOF_CHAR;
    int      VM_SIZEOF_INT;
    int      VM_SIZEOF_SHORT;
    int      VM_SIZEOF_LONG;
    int      VM_SIZEOF_FLOAT;
    int      VM_SIZEOF_DOUBLE;
    int      VM_SIZEOF_LONGDOUBLE;
    int      VM_SIZEOF_VOID_P;
};

extern struct VM vm;
extern void   vm_init (int stack_size);

#endif  /* XCI_VM_H */
