/*
    XCI: eXperimental C Interpreter for ANSI C
    -- Copyright (c) 2000-2001, gondow@jaist.ac.jp
    $Id: xci.y,v 1.9 2002/07/09 11:43:46 gondow Exp $
*/ 

%{
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "AST.h"
#include "xci.h"
#include "symbol.h"
#include "typedef.h"
#include "type.h"
#include "vm.h"
#include "builtin.h"
#include "eval_block.h"
#include "xml.h"

#if  !defined (DEFAULT_XCI_HOME)
#define  DEFAULT_XCI_HOME    "/home/xci/beta1/xci"
#endif /* DEFAULT_XCI_HOME */

/* xci_def.h defines macros to eliminate GCC extensions like __attribute__.
   %H means XCI's home directory, and %F means a given filename.
 */
#if  !defined (DEFAULT_CPP_COMMAND)
#if 0
#define  DEFAULT_CPP_COMMAND \
    "gcc -E -I%H/xci_include -ansi %F"
#endif
#define  DEFAULT_CPP_COMMAND \
    "sed '1s,^\\(#\\!.*\\)$,/* \\1 */,' %F | /usr/bin/cpp -w -D__FILE__=\\\"%F\\\" -I%H/xci_include -ansi"
#endif /* DEFAULT_CPP_COMMAND */

/* When outputing xml-taggled codes, native header files are used. */
#if  !defined (DEFAULT_CPP_COMMAND_FOR_XML)
#define  DEFAULT_CPP_COMMAND_FOR_XML \
    "gcc -E -include %H/xci_def.h -ansi %F"
#endif /* DEFAULT_CPP_COMMAND_FOR_XML */

/* %H will be expanded just in the same way of DEFAULT_CPP_COMMAND (%F not)*/
#if  !defined (DEFAULT_DTD_FILENAME)
#define DEFAULT_DTD_FILENAME	  "ansi_c.dtd"
#endif  /* DEFAULT_DTD_FILENAME */


static void lex_sync (void);
static void print_usage (void);
static void process_options (int argc, char *argv []);
static char *expand_template (char *template, char *xci_home, char *filename);
static void parse_files (void);
static void type_analyze_files (void);
static void print_code (char const *filename, int lineno);
static void output_xml_files (void);
static int  execute (void);

static struct AST **ast_root = NULL; /* array of AST roots */
static struct AST *ast_curr_root;    /* AST root currently being processed */

int         atexit_index = 0;
void        *atexit_func [MAX_ATEXIT_FUNC] ;
char        *filename;	      /* file name currently opened */
char        *inc_filename;    /* file name currently included */
static int  is_typedef = 0;   /* toggle for typedef definition or not */
int         typedef_flag = 1; /* toggle for typedef-name analysis */
/*
  if typedef_flag == 0, the typedef-name analyser always returns IDENTIFER.
  if typedef_flag == 1, the typedef-name analyser *does* typedef-recognition. 
  This is required to redeclare the same name in the inner scope such as:
  typedef int x;
  {
      x x = 99, y = x+1; // the second 'x' must be IDENTIFIER, not TYPEDEF_NAME
  }
 */

/* a structure variable for arguments */
static struct {
    char **file;
    int  file_num;
    char **arg ;
    int  arg_num;
    int  no_cpp;
    int  no_check_main;
    int  is_xml;
    int  stack_size;
    char *xci_home;
    char *cpp_command;
    char *dtd_filename;
} args = {
    NULL,                /* char **file;       */
    0,                   /* int  file_num;     */
    NULL,                /* char **arg ;       */
    0,                   /* int  arg_num;      */
    0,                   /* int  no_cpp;       */
    0,	      	         /* int  no_check_main;*/
    0,                   /* int  is_xml;       */
    1024*512,            /* int  stack_size;   */ /* 512KB */
    DEFAULT_XCI_HOME,    /* char *xci_home;    */
    DEFAULT_CPP_COMMAND, /* char *cpp_command; */
    DEFAULT_DTD_FILENAME /* char *dtd_filename; */
};
%}

/*
  This grammar has 1 shift/reduce conflict in dangling-else.
  The directive '%expect' inhibits the report.
  (I do not like to avoid dangling-else using '%prec THEN'.)
  */
%expect 1

%start translation_unit

/* keywords */
%token AUTO     "auto"
%token BREAK    "break"
%token CASE     "case"
%token CHAR     "char"
%token CONST    "const"
%token CONTINUE "continue"
%token DEFAULT  "default"
%token DO       "do"
%token DOUBLE   "double"
%token ELSE     "else"
%token ENUM     "enum"
%token EXTERN   "extern"
%token FLOAT    "float"
%token FOR      "for"
%token GOTO     "goto"
%token IF       "if"
%token INT      "int"
%token LONG     "long"
%token REGISTER "register"
%token RETURN   "return"
%token SHORT    "short"
%token SIGNED   "signed"
%token STATIC   "static"
%token SIZEOF   "sizeof"
%token STRUCT   "struct"
%token SWITCH   "switch"
%token TYPEDEF  "typedef"
%token UNION    "union"
%token UNSIGNED "unsigned"
%token VOID     "void"
%token VOLATILE "volatile"
%token WHILE    "while"

/* operators */
%token ARROW    "->"
%token EQ       "=="
%token NEQ      "!="
%token LESSEQ   "<="
%token MOREEQ   ">="
%token INC      "++"
%token DEC      "--"
%token AND      "&&"
%token OR       "||"
%token PLUSEQ   "+="
%token MINUSEQ  "-="
%token MULTEQ   "*="
%token DIVEQ    "/="
%token MODEQ    "%="
%token BANDEQ   "&="
%token BEOREQ   "^="
%token BIOREQ   "|="
%token RSHIFTEQ ">>="
%token LSHIFTEQ "<<="
%token RSHIFT   ">>"
%token LSHIFT   "<<"
%token ELLIPSIS "..."

/* identifiers */
%token IDENTIFIER
%token TYPEDEF_NAME

/* literals */
%token STRING
%token INTEGER_CONSTANT
%token CHARACTER_CONSTANT
%token FLOATING_CONSTANT
%token ENUMERATION_CONSTANT

%%
/*
  These rules are introduced only to avoid reduce/reduce conflicts and/or
  shift/reduce conflicts; see bison info "Actions in Mid-Rule".
 */

TD_scope_begin: { typedef_scope_begin (); } ;
TD_scope_end:   { typedef_scope_end ();   } ;
TD_on:          { lex_sync (); typedef_flag = 1; } ;
TD_off:         { lex_sync (); typedef_flag = 0; } ;

/*
  identifiers for name-space objects other than 
   function/variable/enum_constant/typedef_name name-space ones.
   */
identifier
    : IDENTIFIER
    	{ $$ = $1;}
    | TD_off TYPEDEF_NAME TD_on 
    	{ $$ = $2; $$->ast_type = AST_IDENTIFIER; }
    /*
      We need TD_off/TD_on here only to avoid a shift/reduce conflict.
      They are meaningless in the sense that the value of typedef_flag 
      does not change, because TD_off becomes effective right after
      TYPEDEF_NAME is recognized (see lex_sync ()) and next TD_on is done.
     */
    | ENUMERATION_CONSTANT
    	{ $$ = $1; $$->ast_type = AST_IDENTIFIER; }
    ;

/*
  The following is based on the grammar in the book "The C Programming 
  Language, 2nd Edition", Kernighan & Ritchie, Prentice Hall 1998,
  ISBN 0-13-110362-8.
 */

translation_unit
    : external_declaration
	{ $$ = create_AST2 (AST_translation_unit_single,     $1); 
	  ast_curr_root = $$;
	}
    | translation_unit external_declaration
	{ $$ = create_AST3 (AST_translation_unit_pair,       $1, $2);
	  ast_curr_root = $$;
	}
    ;

external_declaration
    : function_definition
	{ $$ = create_AST2 (AST_external_declaration_func,   $1); }
    | declaration
	{ $$ = create_AST2 (AST_external_declaration_dec,    $1); }
    ;

function_definition
    : declaration_specifiers function_declarator
    	TD_on declaration_list_opt compound_statement TD_scope_end
    	{ $$ = create_AST5 (AST_function_definition_spec,    $1, $2, $4, $5); }
    | function_declarator
    	TD_on declaration_list_opt compound_statement TD_scope_end
    	{ $$ = create_AST4 (AST_function_definition_nospec,  $1, $3, $4); }
    ;

declaration_list_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_declaration_list_opt_null); }
    | declaration_list
    	{ $$ = create_AST2 (AST_declaration_list_opt_direct,  $1); }
    ;

declaration
    : declaration_specifiers init_declarator_list_opt TD_on ';'
        { is_typedef = 0;
    	  $$ = create_AST3 (AST_declaration_direct,           $1, $2); }
    ;

init_declarator_list_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_init_declarator_list_opt_null); }
    | init_declarator_list
    	{ $$ = create_AST2 (AST_init_declarator_list_opt_direct, $1); }
    ;

declaration_list
    : declaration
    	{ $$ = create_AST2 (AST_declaration_list_single,       $1); }
    | declaration_list declaration
    	{ $$ = create_AST3 (AST_declaration_list_pair,         $1, $2); }
    ;

declaration_specifiers
    : storage_class_specifier
    	{ $$ = create_AST2 (AST_declaration_specifers_storage,   $1); }
    | storage_class_specifier declaration_specifiers
    	{ $$ = create_AST3 (AST_declaration_specifers_storage_pair, $1, $2); }
    | type_specifier
    	{ $$ = create_AST2 (AST_declaration_specifers_spec,      $1); }
    | type_specifier declaration_specifiers
    	{ $$ = create_AST3 (AST_declaration_specifers_spec_pair, $1, $2); }
    | type_qualifier
    	{ $$ = create_AST2 (AST_declaration_specifers_qual,      $1); }
    | type_qualifier declaration_specifiers
    	{ $$ = create_AST3 (AST_declaration_specifers_qual_pair, $1, $2); }
    ;

storage_class_specifier
    : "auto"       { $$ = create_AST1 (AST_storage_class_specifier_auto);    }
    | "register"   { $$ = create_AST1 (AST_storage_class_specifier_register); }
    | "static"     { $$ = create_AST1 (AST_storage_class_specifier_static);  }
    | "extern"     { $$ = create_AST1 (AST_storage_class_specifier_extern);  }
    | "typedef"    { is_typedef = 1;
    	    	     $$ = create_AST1 (AST_storage_class_specifier_typedef); }
    ;

type_specifier
    : TD_off "void"     { $$ = create_AST1 (AST_type_specifier_void);     }
    | TD_off "char"     { $$ = create_AST1 (AST_type_specifier_char);     }
    | TD_off "short"    { $$ = create_AST1 (AST_type_specifier_short);    }
    | TD_off "int"      { $$ = create_AST1 (AST_type_specifier_int);      }
    | TD_off "long"     { $$ = create_AST1 (AST_type_specifier_long);     }
    | TD_off "float"    { $$ = create_AST1 (AST_type_specifier_float);    }
    | TD_off "double"   { $$ = create_AST1 (AST_type_specifier_double);   }
    | TD_off "signed"   { $$ = create_AST1 (AST_type_specifier_signed);   }
    | TD_off "unsigned" { $$ = create_AST1 (AST_type_specifier_unsigned); }
    | TD_off struct_or_union_specifier
    	     { $$ = create_AST2 (AST_type_specifier_struct_or_union, $2); }
    | TD_off enum_specifier
       	     { $$ = create_AST2 (AST_type_specifier_enum,            $2); }
    | TD_off TYPEDEF_NAME
    	     { $$ = create_AST2 (AST_type_specifier_TYPEDEF_NAME,    $2); }
    /*
      Note: TD_off is done *after* TYPEDEF_NAME is recognized and
            fetched as a look-ahead, which is guaranteed by lex_sync ().
     */
    ;

type_qualifier
    : "const"      { $$ = create_AST1 (AST_type_qualifier_const); }
    | "volatile"   { $$ = create_AST1 (AST_type_qualifier_volatile); }
    ;

struct_or_union_specifier
    : struct_or_union TD_on TD_scope_begin 
    	'{' struct_declaration_list TD_off '}' TD_scope_end
    	{ $$ = create_AST3 (AST_struct_or_union_specifier_notag, $1, $5); }
    | struct_or_union identifier TD_on TD_scope_begin
    	'{' struct_declaration_list TD_off '}' TD_scope_end
    	{ $$ = create_AST4 (AST_struct_or_union_specifier_tag,   $1, $2, $6); }
    | struct_or_union identifier
    	{ $$ = create_AST3 (AST_struct_or_union_specifier_incomplete, $1, $2);}
    ;

struct_or_union
    : "struct"     { $$ = create_AST1 (AST_struct_or_union_struct); }
    | "union"      { $$ = create_AST1 (AST_struct_or_union_union);  }
    ;

struct_declaration_list
    : struct_declaration
    	{ $$ = create_AST2 (AST_struct_declaration_list_single, $1); }
    | struct_declaration_list struct_declaration
    	{ $$ = create_AST3 (AST_struct_declaration_list_pair,   $1, $2); }
    ;

init_declarator_list
    : init_declarator
    	{ $$ = create_AST2 (AST_init_declarator_single,        $1); }
    | init_declarator_list ',' init_declarator
    	{ $$ = create_AST3 (AST_init_declarator_pair,          $1, $3); }
    ;

init_declarator
    : declarator
    	{ $$ = create_AST2 (AST_init_declarator_dec,           $1); }
    | declarator TD_on '=' initializer TD_off
    	{ $$ = create_AST3 (AST_init_declarator_dec_init,      $1, $4); }
    ;

struct_declaration
    : specifier_qualifier_list struct_declarator_list TD_on ';'
    	{ $$ = create_AST3 (AST_struct_declaration_direct,     $1, $2); }
    ;

specifier_qualifier_list
    : type_specifier
    	{ $$ = create_AST2 (AST_specifier_qualifier_list_spec,      $1); }
    | type_specifier specifier_qualifier_list
    	{ $$ = create_AST3 (AST_specifier_qualifier_list_spec_pair, $1, $2); }
    | type_qualifier
    	{ $$ = create_AST2 (AST_specifier_qualifier_list_qual,      $1); }
    | type_qualifier specifier_qualifier_list
    	{ $$ = create_AST3 (AST_specifier_qualifier_list_qual_pair, $1, $2); }
    ;

struct_declarator_list
    : struct_declarator
    	{ $$ = create_AST2 (AST_struct_declarator_list_single, $1); }
    | struct_declarator_list ',' struct_declarator
    	{ $$ = create_AST3 (AST_struct_declarator_list_pair,   $1, $3); }
    ;

struct_declarator
    : declarator
    	{ $$ = create_AST2 (AST_struct_declarator_direct,     $1); }
    | ':' constant_expression
    	{ $$ = create_AST2 (AST_struct_declarator_bitfield_noname, $2); }
    | declarator ':' constant_expression
    	{ $$ = create_AST3 (AST_struct_declarator_bitfield,   $1, $3); }
    ;

enum_specifier
    : "enum" '{' enumerator_list '}' 
    	{ $$ = create_AST2 (AST_enum_specifier_untag,         $3); }
    | "enum" identifier '{' enumerator_list '}'
    	{ $$ = create_AST3 (AST_enum_specifier_tag,           $2, $4); }
    | "enum" identifier
    	{ $$ = create_AST2 (AST_enum_specifier_dec,           $2); }
    ;

enumerator_list
    : enumerator
    	{ $$ = create_AST2 (AST_enumerator_list_single,       $1); }
    | enumerator_list ',' enumerator
    	{ $$ = create_AST3 (AST_enumerator_list_pair,         $1, $3); }
    ;

enumerator
    : IDENTIFIER
    	{ $$ = create_AST2 (AST_enumerator_id,                $1); 
    	  typedef_entry_by_enumerator ($1); }
    | IDENTIFIER TD_on '=' constant_expression TD_off
    	{ $$ = create_AST3 (AST_enumerator_id_init,           $1, $4);
    	  typedef_entry_by_enumerator ($1); }
    	/* follow (enumerator) = { ',' '}' } */
    ;

declarator
    : pointer_opt direct_declarator
    	{ $$ = create_AST3 (AST_declarator_direct,            $1, $2);
	  typedef_entry_by_declarator ($$, is_typedef); }
    ;

pointer_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_pointer_opt_null); }
    | pointer
    	{ $$ = create_AST2 (AST_pointer_opt_direct,           $1); }
    ;    

direct_declarator
    : IDENTIFIER
    	{ $$ = create_AST2 (AST_direct_declarator_id,         $1); }
    | '(' declarator ')'
    	{ $$ = create_AST2 (AST_direct_declarator_paren,      $2); }
    | direct_declarator TD_on '[' constant_expression_opt TD_off ']'
    	{ $$ = create_AST3 (AST_direct_declarator_array,      $1, $4); }
    | direct_declarator TD_scope_begin TD_on 
    	'(' parameter_type_list ')' TD_off TD_scope_end 
    	{ $$ = create_AST3 (AST_direct_declarator_func_new,   $1, $5); }
    | direct_declarator TD_scope_begin TD_on 
    	'(' identifier_list_opt ')' TD_off TD_scope_end 
    	{ $$ = create_AST3 (AST_direct_declarator_func_old,   $1, $5); }
    	/* follow (direct_declarator) = { '[' '(' ')' ';' ',' '=' } */
    ;

/*
  there are not function_declarator, direct_function_declarator in
  K&R 2nd edition; these two rules are introduced to know if a given 
  declarator is a function definition or not in analysing typedef names.
  (if there is a better way to do it when parsing, we would like to 
  eliminate these rules; please let me know!)
  */

function_declarator
    : pointer_opt direct_function_declarator
    	{ $$ = create_AST3 (AST_function_declarator_direct,    $1, $2);
	  typedef_entry_by_function_declarator ($$); }
    ;

direct_function_declarator
    : direct_declarator TD_scope_begin TD_on '(' parameter_type_list ')'
    	{ $$ = create_AST3 (AST_direct_function_declarator_func_new, $1, $5); }
    | direct_declarator TD_scope_begin TD_on '(' identifier_list_opt ')'
    	{ $$ = create_AST3 (AST_direct_function_declarator_func_old, $1, $5); }
    ;

constant_expression_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_constant_expression_opt_null); }
    | constant_expression
    	{ $$ = create_AST2 (AST_constant_expression_opt_direct, $1); }
    ;    

identifier_list_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_identifier_list_opt_null); }
    | identifier_list
    	{ $$ = create_AST2 (AST_identifier_list_opt_direct,   $1); }
    ;    

pointer
    : '*' type_qualifier_list_opt
    	{ $$ = create_AST2 (AST_pointer_single,               $2); }
    | '*' type_qualifier_list_opt pointer
    	{ $$ = create_AST3 (AST_pointer_pair,                 $2, $3); }
    ;

type_qualifier_list_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_type_qualifier_list_opt_nil); }
    | type_qualifier_list
    	{ $$ = create_AST2 (AST_type_qualifier_list_opt_direct, $1); }
    ;

type_qualifier_list
    : type_qualifier
    	{ $$ = create_AST2 (AST_type_qualifier_list_single,   $1); }
    | type_qualifier_list type_qualifier
    	{ $$ = create_AST3 (AST_type_qualifier_list_pair,     $1, $2); }
    ;

parameter_type_list
    : parameter_list
    	{ $$ = create_AST2 (AST_parameter_type_list_direct,   $1); }
    | parameter_list ',' "..."
    	{ $$ = create_AST2 (AST_parameter_type_list_ellipsis, $1); }
    ;

parameter_list
    : parameter_declaration
    	{ $$ = create_AST2 (AST_parameter_list_single,       $1); }
    | parameter_list ',' parameter_declaration
    	{ $$ = create_AST3 (AST_parameter_list_pair,         $1, $3); }
    ;

parameter_declaration
    : declaration_specifiers declarator TD_on
    	{ $$ = create_AST3 (AST_parameter_declaration_dec,   $1, $2); }
    | declaration_specifiers abstract_declarator_opt TD_on
    	{ $$ = create_AST3 (AST_parameter_declaration_abs,   $1, $2); }
    ;

abstract_declarator_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_abstract_declarator_opt_null); }
    | abstract_declarator
    	{ $$ = create_AST2 (AST_abstract_declarator_opt_direct, $1); }
    ;

identifier_list
    : IDENTIFIER
    	{ $$ = create_AST2 (AST_identifier_list_single,      $1); 
    	  typedef_entry_by_identifier_list ($1);                      }
    | identifier_list ',' IDENTIFIER
    	{ $$ = create_AST3 (AST_identifier_list_pair,        $1, $3);
    	  typedef_entry_by_identifier_list ($3);                      }
    ;

initializer
    : assignment_expression
    	{ $$ = create_AST2 (AST_initializer_assign,          $1); }
    | '{' initializer_list '}'
    	{ $$ = create_AST2 (AST_initializer_list,            $2); }
    | '{' initializer_list ',' '}'
    	{ $$ = create_AST2 (AST_initializer_list_comma,      $2); }
    ;

initializer_list
    : initializer
    	{ $$ = create_AST2 (AST_initializer_list_single,      $1); }
    | initializer_list ',' initializer
    	{ $$ = create_AST3 (AST_initializer_list_pair,        $1, $3); }
    ;

type_name
    : specifier_qualifier_list abstract_declarator_opt TD_on
    	{ $$ = create_AST3 (AST_type_name_direct,             $1, $2); }
    ;
    /* follow (type_name) = { ')' } */

abstract_declarator
    : pointer
    	{ $$ = create_AST2 (AST_abstract_declarator_pointer,  $1); }
    | pointer_opt direct_abstract_declarator
    	{ $$ = create_AST3 (AST_abstract_declarator_abs,      $1, $2); }
    ;

direct_abstract_declarator
    : '(' abstract_declarator ')'
    	{ $$ = create_AST2 (AST_direct_abstract_declarator_paren,     $2); }
    | '[' constant_expression_opt ']'
    	{ $$ = create_AST2 (AST_direct_abstract_declarator_array_nil, $2); }
    | direct_abstract_declarator '[' constant_expression_opt ']'
    	{ $$ = create_AST3 (AST_direct_abstract_declarator_array_pair,
			    $1, $3); }
    | '(' parameter_type_list_opt ')'
    	{ $$ = create_AST2 (AST_direct_abstract_declarator_func_nil,  $2); }
    | direct_abstract_declarator '(' parameter_type_list_opt ')'
    	{ $$ = create_AST3 (AST_direct_abstract_declarator_func_pair, $1, $3);}
    ;

parameter_type_list_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_parameter_type_list_opt_null); }
    | parameter_type_list
    	{ $$ = create_AST2 (AST_parameter_type_list_opt_direct,  $1); }
    ;

/*    this rule is solved in xci.l by searching the typedef table.
      see also: typedef.c, typedef.h
TYPEDEF_NAME
    : IDENTIFIER
    ;
 */

statement
    : labeled_statement
    	{ $$ = create_AST2 (AST_statement_labeled,     $1); }
    | expression_statement
    	{ $$ = create_AST2 (AST_statement_expression,  $1); }
    | TD_scope_begin TD_on compound_statement TD_scope_end
    	{ $$ = create_AST2 (AST_statement_compound,    $3); }
    | selection_statement
    	{ $$ = create_AST2 (AST_statement_selection,   $1); }
    | iteration_statement
    	{ $$ = create_AST2 (AST_statement_iteration,   $1); }
    | jump_statement
    	{ $$ = create_AST2 (AST_statement_jump,        $1); }
    ;

labeled_statement
    : identifier ':' statement
    	{ $$ = create_AST3 (AST_labeled_statement_label,     $1, $3); }
    | "case" constant_expression ':' statement
    	{ $$ = create_AST3 (AST_labeled_statement_case,      $2, $4); }
    | "default" ':' statement
    	{ $$ = create_AST2 (AST_labeled_statement_default,   $3); }
    ;

expression_statement
    : expression_opt ';'
    	{ $$ = create_AST2 (AST_expression_statement_direct, $1); }
    ;

expression_opt
    : /* expression */
    	{ $$ = create_AST1 (AST_expression_opt_null); }
    | expression
    	{ $$ = create_AST2 (AST_expression_opt_direct,       $1); }
    ;

/*
  Note: we do not want to use 'declaration_list_opt' here 
  to combine two rules, since it causes 2 shift/reduce conflicts. (sigh..)
  */

compound_statement
    : '{' declaration_list statement_list_opt '}'
    	{ $$ = create_AST3 (AST_compound_statement_decn,     $2, $3); }
    | '{' statement_list_opt '}'
    	{ $$ = create_AST2 (AST_compound_statement_nodecn,   $2); }
    ;

statement_list_opt
    : /* empty */
    	{ $$ = create_AST1 (AST_statement_list_opt_null); }
    | statement_list
    	{ $$ = create_AST2 (AST_statement_list_opt_direct,   $1); }
    ;

statement_list
    : statement
    	{ $$ = create_AST2 (AST_statement_list_single,       $1); }
    | statement_list statement
    	{ $$ = create_AST3 (AST_statement_list_pair,         $1, $2); }
    ;

selection_statement
    : "if" '(' expression ')' statement
    	{ $$ = create_AST3 (AST_selection_statement_if,      $3, $5); }
    | "if" '(' expression ')' statement "else" statement
    	{ $$ = create_AST4 (AST_selection_statement_if_else, $3, $5, $7); }
    | "switch" '(' expression ')' statement
    	{ $$ = create_AST3 (AST_selection_statement_switch,  $3, $5); }
    ;

iteration_statement
    : "while" '(' expression ')' statement
    	{ $$ = create_AST3 (AST_iteration_statement_while,   $3, $5); }
    | "do" statement "while" '(' expression ')' ';'
    	{ $$ = create_AST3 (AST_iteration_statement_do_while, $2, $5); }
    | "for" '(' expression_opt ';' expression_opt ';' expression_opt ')'
       statement
    	{ $$ = create_AST5 (AST_iteration_statement_for,     $3, $5, $7, $9); }
    ;

jump_statement
    : "goto" identifier ';'
    	{ $$ = create_AST2 (AST_jump_statement_goto, $2); }
    | "continue" ';'
    	{ $$ = create_AST1 (AST_jump_statement_continue); }
    | "break" ';'
    	{ $$ = create_AST1 (AST_jump_statement_break); }
    | "return" expression_opt ';'
    	{ $$ = create_AST2 (AST_jump_statement_return, $2); }
    ;

expression
    : assignment_expression
    	{ $$ = create_AST2 (AST_expression_direct, $1); }
    | expression ',' assignment_expression
    	{ $$ = create_AST3 (AST_expression_comma,  $1, $3); }
    ;

assignment_expression
    : conditional_expression
    	{ $$ = create_AST2 (AST_assignment_expression_direct, $1); }
    | unary_expression assignment_operator assignment_expression
    	{ $$ = create_AST4 (AST_assignment_expression_assign, $1, $2, $3); }
    ;


assignment_operator
    : '='         { $$ = create_AST1 (AST_assignment_operator_eq); }
    | "*="        { $$ = create_AST1 (AST_assignment_operator_mult); }
    | "/="        { $$ = create_AST1 (AST_assignment_operator_div); }
    | "%="        { $$ = create_AST1 (AST_assignment_operator_mod); }
    | "+="        { $$ = create_AST1 (AST_assignment_operator_add); }
    | "-="        { $$ = create_AST1 (AST_assignment_operator_sub); }
    | "<<="       { $$ = create_AST1 (AST_assignment_operator_lshift); }
    | ">>="       { $$ = create_AST1 (AST_assignment_operator_rshift); }
    | "&="        { $$ = create_AST1 (AST_assignment_operator_band); }
    | "^="        { $$ = create_AST1 (AST_assignment_operator_beor); }
    | "|="        { $$ = create_AST1 (AST_assignment_operator_bior); }
    ;

conditional_expression
    : logical_or_expression
    	{ $$ = create_AST2 (AST_conditional_expression_direct, $1); }
    | logical_or_expression '?' expression ':' conditional_expression
    	{ $$ = create_AST4 (AST_conditional_expression_cond,   $1, $3, $5); }
    ;

constant_expression
    : conditional_expression
    	{ $$ = create_AST2 (AST_constant_expression_direct, $1); }
    ;

logical_or_expression
    : logical_and_expression
    	{ $$ = create_AST2 (AST_logical_or_expression_direct, $1); }
    | logical_or_expression "||" logical_and_expression
    	{ $$ = create_AST3 (AST_logical_or_expression_lor,    $1, $3); }
    ;

logical_and_expression
    : inclusive_or_expression
    	{ $$ = create_AST2 (AST_logical_and_expression_direct, $1); }
    | logical_and_expression "&&" inclusive_or_expression
    	{ $$ = create_AST3 (AST_logical_and_expression_land,   $1, $3); }
    ;

inclusive_or_expression
    : exclusive_or_expression
    	{ $$ = create_AST2 (AST_inclusive_or_expression_direct, $1); }
    | inclusive_or_expression '|' exclusive_or_expression
    	{ $$ = create_AST3 (AST_inclusive_or_expression_bior,   $1, $3); }
    ;

exclusive_or_expression
    : and_expression
    	{ $$ = create_AST2 (AST_exclusive_or_expression_direct, $1); }
    | exclusive_or_expression '^' and_expression
    	{ $$ = create_AST3 (AST_exclusive_or_expression_beor,   $1, $3); }
    ;

and_expression
    : equality_expression
    	{ $$ = create_AST2 (AST_and_expression_direct,      $1); }
    | and_expression '&' equality_expression
    	{ $$ = create_AST3 (AST_and_expression_band,        $1, $3); }
    ;

equality_expression
    : relational_expression
    	{ $$ = create_AST2 (AST_equality_expression_direct, $1); }
    | equality_expression "==" relational_expression
    	{ $$ = create_AST3 (AST_equality_expression_eq,     $1, $3); }
    | equality_expression "!=" relational_expression
    	{ $$ = create_AST3 (AST_equality_expression_neq,    $1, $3); }
    ;

relational_expression
    : shift_expression
    	{ $$ = create_AST2 (AST_relational_expression_direct, $1); }
    | relational_expression '<' shift_expression
    	{ $$ = create_AST3 (AST_relational_expression_less,   $1, $3); }
    | relational_expression '>' shift_expression
    	{ $$ = create_AST3 (AST_relational_expression_more,   $1, $3); }
    | relational_expression "<=" shift_expression
    	{ $$ = create_AST3 (AST_relational_expression_lesseq, $1, $3); }
    | relational_expression ">=" shift_expression
    	{ $$ = create_AST3 (AST_relational_expression_moreeq, $1, $3); }
    ;

shift_expression
    : additive_expression
    	{ $$ = create_AST2 (AST_shift_expression_direct,    $1); }
    | shift_expression "<<" additive_expression
    	{ $$ = create_AST3 (AST_shift_expression_lshift,    $1, $3); }
    | shift_expression ">>" additive_expression
    	{ $$ = create_AST3 (AST_shift_expression_rshift,    $1, $3); }
    ;

additive_expression
    : multiplicative_expression
    	{ $$ = create_AST2 (AST_additive_expression_direct, $1); }
    | additive_expression '+' multiplicative_expression
    	{ $$ = create_AST3 (AST_additive_expression_add,    $1, $3); }
    | additive_expression '-' multiplicative_expression
    	{ $$ = create_AST3 (AST_additive_expression_sub,    $1, $3); }
    ;

multiplicative_expression
    : cast_expression
    	{ $$ = create_AST2 (AST_multiplicative_expression_direct, $1); }
    | multiplicative_expression '*' cast_expression
    	{ $$ = create_AST3 (AST_multiplicative_expression_mult,   $1, $3); }
    | multiplicative_expression '/' cast_expression
    	{ $$ = create_AST3 (AST_multiplicative_expression_div,    $1, $3); }
    | multiplicative_expression '%' cast_expression
    	{ $$ = create_AST3 (AST_multiplicative_expression_mod,    $1, $3); }
    ;

cast_expression
    : unary_expression
    	{ $$ = create_AST2 (AST_cast_expression_direct,       $1); }
    | '(' type_name ')' cast_expression
    	{ $$ = create_AST3 (AST_cast_expression_cast,         $2, $4); }
    ;

unary_expression
    : postfix_expression
    	{ $$ = create_AST2 (AST_unary_expression_direct,      $1); }
    | "++" unary_expression
    	{ $$ = create_AST2 (AST_unary_expression_inc,         $2); }
    | "--" unary_expression
    	{ $$ = create_AST2 (AST_unary_expression_dec,         $2); }
    | unary_operator cast_expression
    	{ $$ = create_AST3 (AST_unary_expression_unary_op,    $1, $2); }
    | "sizeof" unary_expression
    	{ $$ = create_AST2 (AST_unary_expression_sizeof_exp,  $2); }
    | "sizeof" '(' type_name ')'
    	{ $$ = create_AST2 (AST_unary_expression_sizeof_type, $3); }
    ;

unary_operator
    : '&'      { $$ = create_AST1 (AST_unary_operator_addr);  }
    | '*'      { $$ = create_AST1 (AST_unary_operator_deref); }
    | '+'      { $$ = create_AST1 (AST_unary_operator_add);   }
    | '-'      { $$ = create_AST1 (AST_unary_operator_sub);   }
    | '~'      { $$ = create_AST1 (AST_unary_operator_comp);  }
    | '!'      { $$ = create_AST1 (AST_unary_operator_neg);   }
    ;

postfix_expression
    : primary_expression
    	{ $$ = create_AST2 (AST_postfix_expression_direct,  $1); }
    | postfix_expression '[' expression ']'
    	{ $$ = create_AST3 (AST_postfix_expression_array,   $1, $3); }
    | postfix_expression '(' argument_expression_list ')'
    	{ $$ = create_AST3 (AST_postfix_expression_funcall, $1, $3); }
    | postfix_expression '(' ')'
    	{ $$ = create_AST2 (AST_postfix_expression_funcall_noarg, $1); }
    | postfix_expression '.' identifier
    	{ $$ = create_AST3 (AST_postfix_expression_dot,     $1, $3); }
    | postfix_expression "->" identifier
    	{ $$ = create_AST3 (AST_postfix_expression_arrow,   $1, $3); }
    | postfix_expression "++"
    	{ $$ = create_AST2 (AST_postfix_expression_inc,     $1); }
    | postfix_expression "--"
    	{ $$ = create_AST2 (AST_postfix_expression_dec,     $1); }
    ;

primary_expression
    : IDENTIFIER
    	{ $$ = create_AST2 (AST_primary_expression_ID,       $1); }
    | constant
    	{ $$ = create_AST2 (AST_primary_expression_constant, $1); }
    | STRING
    	{ $$ = create_AST2 (AST_primary_expression_STRING,   $1); }
    | '(' expression ')'
    	{ $$ = create_AST2 (AST_primary_expression_paren,    $2); }
    ;

argument_expression_list
    : assignment_expression    	      
    	{ $$ = create_AST2 (AST_argument_expression_list_single,  $1); }
    | argument_expression_list ',' assignment_expression
    	{ $$ = create_AST3 (AST_argument_expression_list_pair,    $1, $3); }
    ;

constant
    : INTEGER_CONSTANT
    	{ $$ = create_AST2 (AST_constant_INT,   $1); }
    | CHARACTER_CONSTANT
    	{ $$ = create_AST2 (AST_constant_CHAR,  $1); }
    | FLOATING_CONSTANT
    	{ $$ = create_AST2 (AST_constant_FLOAT, $1); }
    | ENUMERATION_CONSTANT
    	{ $$ = create_AST2 (AST_constant_ENUM,  $1); }
    ;
%%
/* ---------------------------------------------------------------------- */
int
yyerror (char *mesg)
{
    fprintf (stderr, "yyerror: at %s (%s): %d\n%s around `%s'\n",
	     inc_filename, filename, lineno, mesg, yytext);
    print_code (inc_filename, lineno);
    exit (1);
}

#if 0
int
yyerror_args (char *mesg, ...)
{
    char     buf [BUFSIZ];
    int      ret_val;

    va_list  pvar;
    va_start (pvar, mesg);
    vsprintf (buf, mesg, pvar);
    ret_val = yyerror (buf);
    va_end (pvar);
    return ret_val;
}
#endif

void
xci_error (struct AST const *ast, char const *mesg)
{
    fprintf (stderr, "xci_error: at %s (%s): %d\n%s\n",
	     ast->inc_filename, ast->filename, ast->lineno, mesg);
    print_code (ast->inc_filename, ast->lineno);
    exit (1);
}

void
xci_error_args (struct AST const *ast, char const *mesg, ...)
{
    char     buf [BUFSIZ];
    va_list  pvar;
    va_start (pvar, mesg);
    vsprintf (buf, mesg, pvar);
    xci_error (ast, buf);
    va_end (pvar);
}

void
xci_error_noast (char *mesg)
{
    fprintf (stderr, "xci_error: %s\n", mesg);
    exit (1);
}

void
xci_error_noast_args (char *mesg, ...)
{
    char     buf [BUFSIZ];
    va_list  pvar;
    va_start (pvar, mesg);
    vsprintf (buf, mesg, pvar);
    xci_error_noast (buf);
    va_end (pvar);
}

void *
emalloc (int size)
{ 
    void *p = malloc (size);
    if (p == NULL) perror ("malloc");
    return p;
}

void *
ecalloc (int nelem, int size)
{ 
    void *p = (void *) calloc (nelem, size);
    if (p == NULL) perror ("calloc");
    return p;
}

void *
erealloc (void *ptr, int size)
{
    void *p = (void *) realloc (ptr, size);
    if (p == NULL) perror ("realloc");
    return p;
}

void
efree (void *ptr)
{
    free (ptr);
}

int
main (int argc, char *argv[])
{
    process_options (argc, argv); /* store them into the structure 'args' */
    parse_files ();               /* create ASTs into 'ast_root []' */
    vm_init (args.stack_size);	  /* initialize the virtual machine */
    register_builtin ();          /* register all built-in functions */
    type_analyze_files ();        /* type analysis and building 'sym_table' */
    if (args.is_xml) {
	output_xml_files ();      /* output xml-tagged codes to files *.xml */
	return 0;
    } else {
	return execute ();	  /* call the function "main" */
    }
}

/* ---------------------------------------------------------------------- */

/*
  to make sure yyparse () does always fetch a look-ahead token eagerly.
  */
static void
lex_sync (void)
{
    if (yychar == YYEMPTY) {
        yychar = yylex ();
        /* just in case yylex returns
           negative to indicate EOF */
        if (yychar < 0) 
            yychar = 0; /* EOF */
    }
#if 0 /* for debugging */
    if (yychar > 128) 
	printf ("yychar = 0d%d\n", yychar );
    else 
	printf ("yychar = %c\n", yychar );
#endif
}

static void
print_usage (void)
{
    fprintf (stderr, 
	     "usage  : xci src1.c src2.c ... [options]\n"
	     "options:\n"
	     "    --args arg1 arg2...  specifies argments\n"
	     "    --no_cpp             inhibits using C preprocessor\n"
	     "    --cpp  cpp_command   overrides XCI_CPP_COMMAND\n"
	     "    --stack_size size    specifies stack size\n"
	     "    --home dir           overrides XCI_HOME\n"
	     "    --xml                output xml-tagged codes to files *.xml\n"
	     "    --dtd_file filename  overrides XCI_DTD_FILENAME\n"
	     "    --no_check_main      inhibits main function checking\n"
	     "    --help               prints this usage\n"
	     );
}

static void
process_options (int argc, char *argv [])
{
    int    i;
    char   *s;

    /* first get environment variables */
    if ((s = getenv ("XCI_HOME")) != NULL) {
	args.xci_home = s;
    }
    if ((s = getenv ("XCI_CPP_COMMAND")) != NULL) {
	args.cpp_command = s;
    }
    if ((s = getenv ("XCI_DTD_FILENAME")) != NULL) {
	args.dtd_filename = s;
    }
    
    for (i = 1; i < argc; i++) {
	if (argv [i][0] != '-') {                    /* filenames */
	    int j;
	    for (j = 0; i < argc && argv [i][0] != '-'; i++, j++) {
		args.file = erealloc (args.file, sizeof (char *) * (j + 1));
		args.file [j] = emalloc (strlen (argv [i]) + 1);
		strcpy (args.file [j], argv [i]);
	    }
	    i--;
	    args.file_num = j;

	    /* put the first filename as "argv [0]" */
	    args.arg = erealloc (args.arg, sizeof (char *));
	    args.arg [0] = emalloc (strlen (args.file [0]) + 1);
	    strcpy (args.arg [0], args.file [0]);
	    args.arg_num = 1;
	} else if (!strcmp (argv [i], "--args")) {    /* arguments */
	    int j;

	    for (i++, j = 1; i < argc && argv [i][0] != '-'; i++, j++) {
		args.arg = erealloc (args.arg, sizeof (char *) * (j + 1));
		args.arg [j] = emalloc (strlen (argv [i]) + 1);
		strcpy (args.arg [j], argv [i]);
	    }
	    i--;
	    args.arg_num = j;
	} else if (!strcmp (argv [i], "--no_cpp")) {
	    args.no_cpp = 1;
	} else if (!strcmp (argv [i], "--cpp")) {
	    i++;
	    if (i > argc) {
		fprintf (stderr, "cpp_command not specified\n");
		print_usage ();
		exit (1);
	    }
	    args.cpp_command = emalloc (strlen (argv [i]) + 1);
	    strcpy (args.cpp_command, argv [i]);
	} else if (!strcmp (argv [i], "--stack_size")) {
	    i++;
	    if (i > argc) {
		fprintf (stderr, "stack size not specified\n");
		print_usage ();
		exit (1);
	    }
	    args.stack_size = atoi (argv [i]);
	} else if (!strcmp (argv [i], "--no_check_main")) {
	    args.no_check_main = 1;
	} else if (!strcmp (argv [i], "--xml")) {
	    args.is_xml = 1;
	    /* '--xml' implies '--no_check_main' */
	    args.no_check_main = 1;
	} else if (!strcmp (argv [i], "--dtd_file")) {
	    i++;
	    if (i > argc) {
		fprintf (stderr, "dtd_file not specified\n");
		print_usage ();
		exit (1);
	    }
	    args.dtd_filename = emalloc (strlen (argv [i]) + 1);
	    strcpy (args.dtd_filename, argv [i]);
	} else if (!strcmp (argv [i], "--help")) {
	    print_usage ();
	    exit (1);
	} else {
	    /* unknown option */
	    fprintf (stderr, "unknown option: %s\n", argv [i]);
	    print_usage ();
	    exit (1);
	}
    }

    if (args.file_num <= 0) {
	fprintf (stderr, "error: no file specified\n");
	print_usage ();
	exit (1);
    }

    if (args.is_xml) {
	/* native header files are used for xml-tagging */
	args.cpp_command = DEFAULT_CPP_COMMAND_FOR_XML;
    }

    args.dtd_filename = expand_template (args.dtd_filename, args.xci_home, "");
}

/*
  %H, %F in template are expanded with xci_home and filename.
  the expanded string is stored in the buffer pointed to by cpp_command.
  expand_template returns a pointer cpp_command.

  e.g., "gcc -E -include %H/xci_def.h -ansi %F"
        => "gcc -E -include /gondow/xci/src/xci_def.h -ansi hello.c"
 */
static char *
expand_template (char *template, char *xci_home, char *filename)
{
    char *cpp_command;
    char *from, *to;
    int  num_h = 0, num_f = 0, size;

    /* first count %H, %F */
    for (from = template; *from != '\0'; from++) {
	if (*from == '%') {
	    from++;
	    switch (*from) {
	    case 'H': num_h++;    break;
	    case 'F': num_f++;    break;
	    default:  assert (0); break;
	    }
	}
    }

    from = template;
    size = (strlen (template) + 1) + (strlen (xci_home) + 1) * num_h
	   + (strlen (filename) + 1) * num_f;
    to   = cpp_command = malloc (size);

    while (*from != '\0') {
	if (*from == '%') {
	    from++;
	    switch (*from) {
	    case 'H':
		strcpy (to, xci_home);
		to += strlen (xci_home);
		break;
	    case 'F':
		strcpy (to, filename);
		to += strlen (filename);
		break;
	    default:
		assert (0);
		break;
	    }
	    from++;
	} else {
	    *to++ = *from++;
	}
    }
    *to = '\0';
    return cpp_command;
}

static void
parse_files ()
{
    int  i;
    char *cpp_command;
    FILE *fp;

    for (i = 0; i < args.file_num; i++) {
	filename = args.file [i];

	ast_root = realloc (ast_root, sizeof (struct AST *) * (i + 1));

	if (args.no_cpp) {
	    fp = fopen (filename, "r");
	} else {
	    cpp_command = expand_template (args.cpp_command,
					   args.xci_home, filename); 
	    fp = popen (cpp_command, "r");
	}

	if (fp == NULL) {
	    fprintf (stderr, "cannot open: %s\n", filename);
	    exit (1);
	}

	yyin = fp;
	inc_filename = filename;

	/* parsing & constructing an abstract syntax tree */
	typedef_scope_begin ();
	yyparse ();
	typedef_scope_end ();

	ast_root [i] = ast_curr_root;

	if (args.no_cpp) {
	    fclose (fp);
	} else {
	    pclose (fp);
	}
    }
}

static void
type_analyze_files (void)
{
    int   i;

    for (i = 0; i < args.file_num; i++) {
	sym_begin_file (args.file[i]);
	type_analyze (ast_root [i]);
	sym_end_file ();
    }
    if (!args.no_check_main) {
	check_main_type ();
    }
}

static void
print_code (char const *filename, int lineno)
{
    static char *display_command = "cat -n %s | head -%d | tail -3";
    char *buf;

    buf = emalloc (strlen (display_command) + strlen (inc_filename) + 20);
    sprintf (buf, display_command, inc_filename, lineno+1);
    system (buf);
}

static void
output_xml_files (void)
{
    int  i;
    for (i = 0; i < args.file_num; i++) {
	output_xml (args.file [i], args.dtd_filename, ast_root [i]);
    }
}

static int
execute (void)
{
    struct Symbol *sym;
    void   *old_sp;
    int    i, arg_num, ret_val, ret_size;

    sym = sym_lookup_at_all_files (MAIN_NAME, NS_NORMAL);
    assert (sym != NULL);
    assert (sym->type->kind == TYPE_KIND_FUNCTION);

    vm.status = VM_STAT_RUNNING;
    arg_num = sym->type->u.t_function.arg_num;
    ret_size = sym->type->u.t_function.ret_type->size;
    assert (ret_size == vm.VM_SIZEOF_INT);
    old_sp = vm.get_sp ();
    if (arg_num == 2) {
	vm.push_p (args.arg);
	vm.push_i (args.arg_num);
    }
    vm.push_i (0);    /* space for return value */
    vm.push_p (NULL);	 /* return address (NULL means nowhere to return) */
    eval_block (sym->addr);

    /* copy the return value of main */
    vm.top_b (old_sp, ret_size);

    /* discard argc and argv, if exists */
    vm.set_sp (old_sp + ret_size);

    ret_val = vm.pop_i ();

    /* call functions registered by atexit() */
    for (i = 0; i < MAX_ATEXIT_FUNC; i++) {
	struct AST *ast = atexit_func [i];
	if (ast == NULL)
	    break;
	vm.push_p (NULL); /* return address */
	eval_block (ast);
    }

    vm.status = VM_STAT_STOPPED;
    return ret_val;
}
