/*
    XCI: Experimental C Interpreter for ANSI C
    -- Copyright (c) 2000-2002, gondow@jaist.ac.jp
    $Id: stdio.h,v 1.7 2002/07/09 11:43:47 gondow Exp $
*/ 

#ifndef XCI_STDIO_H
#define XCI_STDIO_H

#include <stddef.h> /* for size_t, NULL */
#include <stdarg.h>
#define EOF  (-1)
#define BUFSIZ 1024
#define _IOFBF 0
#define _IOLBF 1
#define _IONBF 2
#define FOPEN_MAX 20
#define FILENAME_MAX (260-1)
#define L_tmpnam FILENAME_MAX
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#define TMP_MAX 26

typedef long fpos_t;
typedef struct FILE FILE;

extern FILE *stdin, *stdout, *stderr;

int printf  (char const *format, ...);
int fprintf (FILE *fp, char const *format, ...);
int sprintf (char *s, char const *format, ...);

int vprintf (char const *format, va_list arg);
int vfprintf (FILE *stream, char const *format, va_list arg);
int vsprintf (char *s, char const *format, va_list arg);

FILE *fopen (const char *filename, const char *mode);
FILE *freopen (const char *filename, const char *mode, FILE *stream);
int fflush  (FILE *stream);
int fclose  (FILE *fp);
int fputs   (const char *s, FILE *fp);
char *fgets (char *buf, int n, FILE *fp);

int fgetc   (FILE *fp);
#define getc(fp) fgetc(fp)
#define getchar() getc(stdin)
int fputc   (int c, FILE *fp);
#define putc(c, fp) fputc(c, fp)
#define putchar(c)  putc(c, stdout)
int ungetc (int c, FILE *stream);

int remove  (const char *filename);
int rename  (const char *oldname, const char *newname);
FILE *tmpfile (void);
char *tmpnam (char *s); 
int setvbuf (FILE *stream, char *buf, int mode, size_t size);
void setbuf (FILE *stream, char *buf);

size_t fread (void *ptr, size_t size, size_t nobj, FILE *stream);
size_t fwrite (const void *ptr, size_t size, size_t nobj, FILE *stream);

long ftell   (FILE *stream);
void rewind  (FILE *stream);
int  fseek   (FILE *stream, long offset, int origin);
int  fgetpos (FILE *stream, fpos_t *ptr);
int  fsetpos (FILE *stream, const fpos_t *ptr);

void clearerr (FILE *stream);
int  feof     (FILE *stream);
int  ferror   (FILE *stream);
void perror   (const char *s);

int  scanf    (char const *format, ...);
int  fscanf   (FILE *stream, char const *format, ...);
int  sscanf   (char const *s, char const *format, ...);
#endif /* XCI_STDIO_H */
