/*
    XCI: Experimental C Interpreter for ANSI C
    -- Copyright (c) 2000-2002, gondow@jaist.ac.jp
    $Id: stdlib.h,v 1.2 2002/07/09 11:43:47 gondow Exp $
*/ 

#ifndef XCI_STDLIB_H
#define XCI_STDLIB_H

#include <stddef.h> /* for size_t, NULL */

typedef struct {int  quot, rem; }  div_t;
typedef struct {long quot, rem; } ldiv_t;

#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1
#define RAND_MAX 32767
#define MB_CUR_MAX  /* not implemented */

double atof   (const char *nptr);
int    atoi   (const char *s);
long   atol   (const char *nptr);
double strtod (const char *nptr, char **endptr);
long   strtol (const char *nptr, char **endptr, int base);
unsigned long strtoul (const char *nptr, char **endptr, int base);

int  rand  (void);
void srand (unsigned seed);

void *calloc (size_t nmemb, size_t size);
void free (void *ptr);
void *malloc (size_t size);
void *realloc (void *ptr, size_t size);

void abort (void);
int atexit (void (*func)(void));
void exit  (int status);
char *getenv (const char *name);
int system (const char *string);
void *bsearch (const void *key, const void *base, size_t nmemb, size_t size,
	       int (*compar)(const void *, const void *));
void qsort (void *base, size_t nmemb, size_t size,
	    int (*compar)(const void *, const void *));
int    abs  (int j);
long   labs (long j);
div_t  div  (int  number, int  denom);
ldiv_t ldiv (long number, long denom);

/* not implemented */
int mblen (const char *s, size_t n);
int mbtowc (wchar_t *pwc, const char *s, size_t n);
int wctomb (char *s, wchar_t wchar);
size_t mbstowcs (wchar_t *pwcs, const char *s, size_t n);
size_t wcstombs (char *s, const wchar_t *pwcs, size_t n);
#endif /* XCI_STDLIB_H */

